/*
    Copyright (C) 2001 by Christopher Nelson

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"
//#include "cssys/sysfunc.h"
#include "csutil/cscolor.h"
#include "csutil/csevent.h"
#include "csutil/event.h"
#include "cstool/csfxscr.h"
#include "cstool/csview.h"
#include "cstool/initapp.h"
#include "cstool/cspixmap.h"
#include "csutil/cmdhelp.h"
#include "csutil/csinput.h"
#include "csgeom/quaterni.h"
#include "csgeom/matrix3.h"
#include "ivideo/graph3d.h"
#include "ivideo/fontserv.h"
#include "ivideo/graph2d.h"
#include "ivideo/txtmgr.h"
#include "ivideo/natwin.h"
#include "ivaria/conout.h"
//#include "ivaria/dynamics.h"
#include "iengine/engine.h"
#include "iengine/sector.h"
#include "iengine/light.h"
#include "iengine/camera.h"
#include "iengine/campos.h"
#include "iengine/mesh.h"
//#include "iengine/statlght.h"
#include "iengine/movable.h"
#include "iengine/halo.h"
// Comment the 2 below if your copy of CS is older than Feb 22, 2003
#include "imesh/thing.h"
#include "ivideo/polyrender.h"
// Uncomment the 2 below if your copy of CS is older than Feb 22, 2003
//#include "imesh/thing/thing.h"
//#include "imesh/thing/polygon.h"
#include "imesh/particle.h"
#include "imesh/sprite2d.h"
#include "imesh/sprite3d.h"
#include "imesh/spritecal3d.h"
#include "imesh/ball.h"
#include "imesh/object.h"
#include "imap/reader.h"
#include "igraphic/imageio.h"
#include "imap/parser.h"
#include "iengine/material.h"
#include "ivaria/reporter.h"
#include "iutil/event.h"
#include "iutil/eventq.h"
#include "iutil/virtclk.h"
#include "iutil/plugin.h"
#include "iutil/vfs.h"
#include "qsqrt.h"
//#include "iostream"

#include "inetwork/vosa3dl.h"
#include "inetwork/vosapi.h"

#include "csgeom/csrect.h"
#include "csgeom/csrectrg.h"

#include "vos/vos/vos.hh"
#include "vos/metaobjects/a3dl/model.hh"
#include "vos/metaobjects/a3dl/object3d.hh"
#include "vos/metaobjects/misc/factory.hh"
#include "vos/metaobjects/misc/avatar.hh"
#include "vos/metaobjects/misc/talkative.hh"
#include "vos/metaobjects/a3dl/viewpoint.hh"
#include "vos/metaobjects/a3dl/actor.hh"

#include "betaverse.h"
#include "net.h"
#include "parser.h"
#include "paws/pawsmanager.h"
#include "mainwidget.h"
#include "chat.h"
#include "contact.h"

#include <stdio.h>
#include <stdlib.h>
//#include "inetwork/sockerr.h"
csRef<iFile> input;

const int BetaVerse_MULTIPROC  = 0;
const int BetaVerse_SINGLEPROC = 1;
const int BetaVerse_SCREEN     = 2;

const int BetaVerse_CANVAS=BetaVerse_SCREEN;
// @@@@@ engine view as window background doesn't work with the proctex canvases
// @@@@@ yes it does! it just doesn't update right.
//-----------------------------------------------------------------------------

#define  QUERY_REG(myPlug, iFace, errMsg) \
  myPlug = CS_QUERY_REGISTRY (object_reg, iFace); \
  if (!myPlug) \
  { \
    Report (CS_REPORTER_SEVERITY_ERROR, errMsg); \
    return false; \
  }

extern BetaVerse *System;
extern ParserData pdata;

using namespace VOS;

BetaVerse::BetaVerse()
{
  message[0] = 0;
  playerRun = 0;
  running = runningBack = turnRight = turnLeft = false;
  camForward = camBackward = camRight = camLeft = false;
  
}

BetaVerse::~BetaVerse()
{
}

static bool AwsEventHandler( iEvent& ev )
{
  if( ev.Type == csevBroadcast && ev.Command.Code == cscmdProcess )
  {
    System->SetupFrame();
    return true;
  }
  else if( ev.Type == csevBroadcast && ev.Command.Code == cscmdFinalProcess )
  {
    System->FinishFrame();
    return true;
  }
  else
  {
    return System ? System->HandleEvent( ev ) : false;
  }
}

bool BetaVerse::Initialize( int argc, const char* const argv[], const char *iConfigName )
{
  object_reg = csInitializer::CreateEnvironment( argc, argv );

  if( !object_reg )
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "Could not init app, object_reg failure!" );
    return false;
  }

  if( !csInitializer::SetupConfigManager( object_reg, "/this/config/beta.cfg" ))
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "Could not init app, Setup Config failure!" );
    return false;
  }

  if( !BetaVerse::ReadScript( object_reg, "/this/config/local.lst" ))
  {
    Report( CS_REPORTER_SEVERITY_WARNING, "Could not read local list, ReadScript failure! Trying default list. BetaVerse::ReadScript()" );

    if(!BetaVerse::ReadScript( object_reg, "/this/config/default.lst" ))
    {
      Report( CS_REPORTER_SEVERITY_ERROR, "Could not read default list, ReadScript failure! BetaVerse::ReadScript()" );
    }
  }

#ifdef BV_DEBUG
  printf( "*** DEBUG ***\n" );
  printf( "*** After Readscript exits ***\n" );
  printf( "*** pdata->name: %s\n", pdata.name );
  printf( "*** pdata->irc_srv: %s\n", pdata.irc_srv );
  printf( "*** END DEBUG ***\n" );
#endif

  if( !csInitializer::RequestPlugins( object_reg, CS_REQUEST_END ))
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "Could not init app, Requesting Plugins failure!" );
    return false;
  }

  if( !csInitializer::SetupEventHandler( object_reg, AwsEventHandler ))
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "Could not setup event handler!" );
    return false;
  }
  
  // Check for commandline help.
  if( csCommandLineHelper::CheckHelp( object_reg ))
  {
    csCommandLineHelper::Help( object_reg );
    exit (0);
  }
#if 0
  dyn = CS_QUERY_REGISTRY( object_reg, iDynamics );
  if( !dyn )
  {
    csReport( object_reg, CS_REPORTER_SEVERITY_ERROR,
        "crystalspace.application.phystut",
        "No iDynamics plugin!" );
    return false;
  }
#endif
  // The virtual clock.
  vc = CS_QUERY_REGISTRY( object_reg, iVirtualClock );
  csRef<iPluginManager> plugin_mgr(
  	CS_QUERY_REGISTRY( object_reg, iPluginManager ));

  // Load the engine plugin.
  Report( CS_REPORTER_SEVERITY_NOTIFY, "Loading engine..." );
  engine = CS_LOAD_PLUGIN( plugin_mgr, "crystalspace.engine.3d", iEngine );
  if( !engine )
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "Could not load the engine plugin!" );
    return false;
  }
  if( !object_reg->Register( engine, "iEngine" ))
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "Could not register engine!" );
    return false;
  }

  QUERY_REG( g3d, iGraphics3D, "Couldn't load iGraphics3D plugin!" );
  QUERY_REG( g2d, iGraphics2D, "Couldn't load iGraphics2D plugin!" );
  QUERY_REG( VFS, iVFS, "Couldn't load iVFS plugin!" );
  QUERY_REG( myConsole, iConsoleOutput, "Couldn't load iConsoleOutput plugin!" );

  kbd = CS_QUERY_REGISTRY( object_reg, iKeyboardDriver );
  if( kbd == NULL )
  {
    csReport( object_reg, CS_REPORTER_SEVERITY_ERROR,
      "crystalspace.application.betaverse",
      "No iKeyboardDriver plugin!" );
    return false;
  }

  loader = CS_LOAD_PLUGIN( plugin_mgr, "crystalspace.level.loader", iLoader );
  if( !loader )
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "No iLoader plugin!" );
    return false;
  }
  
  vosa3dl = CS_QUERY_REGISTRY (object_reg, iVosA3DL);
  if( !vosa3dl )
  {
    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
        "crystalspace.application.betaverse",
        "No iVosA3DL plugin!");
    return false;
  }
  
  csRef<iVosApi> vosapi = SCF_QUERY_INTERFACE(vosa3dl, iVosApi);
  if( !vosapi )
  {
    csReport( object_reg, CS_REPORTER_SEVERITY_ERROR,
        "crystalspace.application.betaverse",
	"No VOS api!" );
    return false;
  }
  
  
    
  // Create the ball mesh factory.
  ballFact = engine->CreateMeshFactory("crystalspace.mesh.object.ball",
   "ballFact");
  if (ballFact == 0)
  {
    csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
        "crystalspace.application.betaverse",
        "Error creating mesh object factory!");
    return false;
  }
  
  // Open the main system. This will open all the previously loaded plug-ins.
  iNativeWindow* nw = g2d->GetNativeWindow();
  if( nw )
    nw->SetTitle( "BetaVerse v0.2 alpha" );

  if( !csInitializer::OpenApplication( object_reg ))
  {
    Report( CS_REPORTER_SEVERITY_ERROR, "Error opening system!" );
    return false;
  }
  // Mount some dirs... --mason
  VFS->Mount( "/lib/beta/chars/", "$.$/data$/chars.zip" );
  VFS->Mount( "/lib/beta/tex/", "$.$/data$/tex.zip" );
  VFS->Mount( "/lib/beta/wrld", "$.$/data$/bv.zip" );
  VFS->Mount( "/lib/beta/player", "$.$/data$/player.zip" );
  VFS->Mount( "/lib/beta/config", "$.$/config$/" );
  VFS->Mount( "/fonts/ttf/", "$.$/config/ttf$/" );
  VFS->Mount( "/tmp/", "/tmp$/" );

  loading = LoadPixmap( "/lib/beta/tex/loadbverse.jpg" );
  g3d->BeginDraw( CSDRAW_2DGRAPHICS );
  int w = g2d->GetWidth();
  int h = g2d->GetHeight();
  int x1 = ( w/2 ) - 400;
  int x2 = x1 + 800;
  int y1 = ( h/2 ) - 300;
  int y2 = y1 + 600;
  loading->DrawScaled( g3d, x1, y1, x2, y2 );
  g3d->FinishDraw();
  g3d->Print( NULL );
  delete loading;

  centerX = w/2;
  centerY = h/2;
  // Setup the texture manager
  txtmgr = g3d->GetTextureManager ();
  //txtmgr->SetPalette();
  //txtmgr->SetVerbose (true);

  font = g2d->GetFontServer()->LoadFont( CSFONT_LARGE );

  CS_ASSERT(font);
  
  // Initialize the console
  if( myConsole != NULL )
    // Don't let messages before this one appear
    myConsole->Clear();

  // Have to look into the LightingCacheModes -- mason
  engine->SetLightingCacheMode( 0 );
  
  pawsMan = new PawsManager( object_reg );
  pawsMain = new MainWidget( pawsMan );
  //pawsMain->SetCamera( c );
  pawsMan->SetMainWidget( pawsMain );
  
  DeclareBvFactories();
  LoadPawsWidgets();
  
  chat = (ChatWindow *)pawsMain->FindWidget( "ChatBox" );
  contact = (ContactWindow *)pawsMan->FindWidget( "ContactLists" );
  //keys = (KeysWindow *)pawsMain->FindWidget( "KeysBox" );
  
  CS_ASSERT( contact );
  CS_ASSERT( chat );
  //CS_ASSERT( keys );
  
  contact->Init( this );
  contact->SetAlwaysOnTop( true );
  chat->Init( this );

  if( contact )
    contact->Show();  
  if (chat)
    chat->Show();
      
  printf( "Starting the VOS shit\n" );
  //Get local site.
  site = vosapi->GetVobject()->getSite();
  CS_ASSERT( site );
  try
  {
    site->addType( "misc:talkative" );
    site->setDefaultPolicy( "core:read-only" );
  }
  catch( std::exception &e )
  {
    LOG( "BetaVerse::Init()", 0, "Error with local site: " << e.what() );
  }
      
  ChatListener *cl = new ChatListener( chat );
  
  vRef<LocalTalkative> talk = meta_cast<LocalTalkative>( site );
  try
  {
    talk->setDefaultPolicy( "talkative:send-only" );
    talk->addTalkListener( cl );
  }  
  catch( std::exception &e )
  {
    LOG( "BeteaVerse::Init()", 0, "Problem with talk: " << e.what() );
  }
 
  //Get the vos sector, then display it...
  view = csPtr<iView> (new csView (engine, g3d));
  view->GetCamera ()->SetSector (engine->CreateSector("_tmp"));
  view->GetCamera ()->GetTransform ().SetOrigin (csVector3 (0, 5, -3));
  iGraphics2D* g2d = g3d->GetDriver2D ();
  view->SetRectangle (0, 0, g2d->GetWidth (), g2d->GetHeight ());

  if (!g3d->BeginDraw (engine->GetBeginDrawFlags () | CSDRAW_3DGRAPHICS | CSDRAW_CLEARZBUFFER | CSDRAW_CLEARSCREEN));
  view->Draw ();
  char* vosWorldURL = "vop://cyberlake.dyndns.biz/world";
  csRef<iVosSector> vossector = vosa3dl->GetSector(vosWorldURL);
  vossector->Load();
  view->GetCamera()->SetSector(vossector->GetSector());
  vossector->IncRef();
  
  csRef<iVosApi> iSector = SCF_QUERY_INTERFACE( vossector, iVosApi );
  vRef<Vobject> obj = iSector->GetVobject();
  
  //Get remote site... --mason
  remoteSite = obj->getSite();
  
  printf( "Types for obj:\n" );
  TypeSetIterator it = obj->getTypes();
  const char* type;
  for( it; it.hasMore(); it++ )
  {
    type = (*it).c_str();
    printf( "%s\n", type );
  }
    
  vRef<CompoundFactory> cFact = meta_cast<CompoundFactory>( iSector->GetVobject() );
  if( !cFact )
  {
       csReport (object_reg, CS_REPORTER_SEVERITY_ERROR,
        "crystalspace.application.betaverse",
        "Error creating compound vobject factory!");
    return false;
  } 
  try
  {
    //printf( "Construct\n" );
    vRef<Vobject> vObj = cFact->construct( pdata.nick, "misc:avatar" );
    //printf( "Meta Cast\n" );
    vRef<Avatar> pAvatar = meta_cast<Avatar>( vObj );
    //const std::string nick = pdata.nick;
    pAvatar->setNick( pdata.nick );
    playerModel = meta_cast<A3DL::Model>( vObj );
  }
  catch( std::exception& e )
  {
    LOG( "betaverse-Init", 0, "Error using cFact:" << e.what()) ;
    printf( "Couldn't Use Factory\n" );
  }
  
  vRef<Talkative> remoteTalk = meta_cast<Talkative>( playerModel );
  try
  {
    remoteTalk->addRelay( talk );
  }
  catch( std::exception &e )
  {
    LOG( "BetaVerse::Init()", 0, "Couldn;t add local to remote Talkative: " << e.what() );
  }

  double x, y, z;
  x = y = z = 0.0;  
  try
  {
    vRef<Vobject> obj = iSector->GetVobject()->getSite()->findObject( "CameraStart" );
    if( !obj )                                                
      LOG("betaverse", 0, "No Default Camera object" );
    vRef<A3DL::Viewpoint> Cam = meta_cast<A3DL::Viewpoint>( obj );
    if( !Cam )
      LOG("betaverse", 0, "No Obj3D Default Camera object" );
    Cam->getPosition( x, y, z );
  }
  catch( std::exception &e )
  {
    LOG( "betaverse-Init", 0, "Error getting default camera position: " << e.what() );
  }  
  
  view->GetCamera()->GetTransform().SetOrigin( csVector3( x, y, z ) );
  
  ircData = new csString();
  
  bNet = new BetaNet();

  // WHM
  // This is now set from local.lst
  printf( "pdata->irc_srv: %s\n", pdata.irc_srv );
  if( !bNet->Init( object_reg, pdata.irc_srv, 6667, this ))
  {
    csReport( object_reg, CS_REPORTER_SEVERITY_ERROR,
      "crystalspace.application.betaverse",
      "Can't Init BetaNet" );
    return false;
  }

  char msg[255];
  sprintf( msg, "NICK %s\n", pdata.nick );
  printf( msg );
  bNet->SendIRC( msg );
  printf( pdata.usr );
  sprintf( msg, "USER %s\n", pdata.usr );
  printf( msg );
  bNet->SendIRC( msg );
  sprintf( msg, "JOIN %s\n", pdata.ChannelData.chan );
  printf( msg );
  bNet->SendIRC( msg );
  
  roommates = new csStringArray( 128, 128 );
  buddies = new csStringArray( 128, 128 );
  counter = 0;
  mouseLook = true;
  
  g2d->SetMousePosition( centerX, centerY );
  engine->Prepare();
  
  Report( CS_REPORTER_SEVERITY_NOTIFY, "BetaVerse Init done." );
  
  return true;
}

void BetaVerse::SetupFrame()
{
  
  // First get elapsed time from the virtual clock.
  csTicks elapsed_time = vc->GetElapsedTicks ();
  const float speed = elapsed_time / 1000.0;
  // Tell 3D driver we're going to display 3D things.
  if( !g3d->BeginDraw( engine->GetBeginDrawFlags() | CSDRAW_3DGRAPHICS | 
      CSDRAW_CLEARZBUFFER | CSDRAW_CLEARSCREEN ))
    return;
  
  iCamera* c = view->GetCamera();
  // Move the Camera around --mason
  if( camRight )
    rotY += speed;
  if( camLeft )
    rotY -= speed;
  if( camForward )
    rotX += speed;
  if( camBackward )
    rotX -= speed;
  if( camUp )
    c->Move( CS_VEC_FORWARD * 4 * speed );
  if( camDown )
    c->Move( CS_VEC_BACKWARD * 4 * speed );
  
  csMatrix3 rot =  csXRotMatrix3(rotX) * csYRotMatrix3(rotY);
  csOrthoTransform ot(rot, c->GetTransform().GetOrigin());
  c->SetTransform(ot);
    
  // Garb pointers & test for Validity --mason
  vRef<A3DL::Object3D> obj;
  csVector3 pos;
  double x, y, z;
  x = y = z = 0.0;
  
  if( playerModel.isValid() )
  {
    try
    {
      obj = meta_cast<A3DL::Object3D>( playerModel );
      obj->getPosition( x, y, z );
    }
    catch( std::exception &e )
    {
      LOG( "BetaVerse::SetupFrame()", 0, "Error with object3d: " << e.what() );
    } 
  }
  pos = csVector3( x, y, z );
  
  if( running )
  {
    try
    {
      double ox, oy, oz, phi = 0;
      obj->getOrientationAsQuaternion( ox, oy, oz, phi );
      csQuaternion q = csQuaternion( phi, ox, oy, oz );
      csMatrix3 m = csMatrix3( q );
      csReversibleTransform rt = csReversibleTransform( m, csVector3( 0 ) );
      csVector3 move = rt.Other2This( csVector3( 0, 0, 0.25 ) );
      pos += move;              
      obj->setPosition( pos.x, pos.y, pos.z );
    }
    catch( std::exception &e )
    {
      LOG( "BetaVerse::SetupFrame()", 0, "Failed to move object3d: " << e.what() );
    }
  }
  
  if( runningBack )
  {
    try
    {
      double ox, oy, oz, phi = 0;
      obj->getOrientationAsQuaternion( ox, oy, oz, phi );
      csQuaternion q = csQuaternion( phi, ox, oy, oz );
      csMatrix3 m = csMatrix3( q );
      csReversibleTransform rt = csReversibleTransform( m, csVector3( 0 ) );
      csVector3 move = rt.Other2This( csVector3( 0, 0, -0.25 ) );
      pos += move;              
      obj->setPosition( pos.x, pos.y, pos.z );
    }
    catch( std::exception &e )
    {
      LOG( "BetaVerse::SetupFrame()", 0, "Failed to move object3d: " << e.what() );
    }
  }
  
  if( turnRight )
  {
    try
    {
      double ox, oy, oz, phi;
      obj->getOrientation( ox, oy, oz, phi );
      obj->setOrientation( ox, oy, oz, phi + 12.5 );
    }
    catch( std::exception &e )
    {
      LOG( "betaverse.SetupFrame()", 0, "Problems seting orientation: " << e.what() );
    }
  }
  
  if( turnLeft )
  {
    try
    {
      double ox, oy, oz, phi;
      obj->getOrientation( ox, oy, oz, phi );
      obj->setOrientation( ox, oy, oz, phi - 12.5 );
    }
    catch( std::exception &e )
    {
      LOG( "betaverse.SetupFrame()", 0, "Problems seting orientation: " << e.what() );
    }
  }   

  // Tell the camera to render into the frame buffer.
  view->Draw();


  counter++;
  if( counter >= 5 )
  {
    counter = 0;
    //bNet->Parse();
  }
  
  g3d->BeginDraw(engine->GetBeginDrawFlags() | CSDRAW_2DGRAPHICS );
  pawsMan->Draw();
}

void BetaVerse::FinishFrame()
{
  g3d->FinishDraw();
  g3d->Print( NULL );
}

bool BetaVerse::HandleEvent( iEvent &Event )
{
  if( pawsMan->HandleEvent( Event ) )
    return true;  
    
  
  vRef<A3DL::Object3D> obj;
  vRef<A3DL::Actor> act;
  csVector3 pos;
  double x, y, z;
  x = y = z = 0.0;
  
  if( playerModel.isValid() )
  {
    try
    {
      obj = meta_cast<A3DL::Object3D>( playerModel );
      obj->getPosition( x, y, z ); 
    }
    catch( std::exception &e )
    {
      LOG( "BetaVerse::HandleEvent()", 0, "Error with obj3d: " << e.what() );
    }
    try
    {
      act = meta_cast<A3DL::Actor>( playerModel );
    }
    catch( std::exception &e )
    {
      LOG( "BetaVerse::HandleEvent()", 0, "Error with act: " << e.what() );
    }      
  }
  pos = csVector3( x, y, z );
         	    
  switch( csKeyEventHelper::GetEventType (&Event) )
  {
    case csKeyEventTypeDown:
    {  
      switch( csKeyEventHelper::GetCookedCode( &Event ))
      {
        case CSKEY_ESC:
        {
          csRef<iEventQueue> q( CS_QUERY_REGISTRY( object_reg, iEventQueue ));
          if( q )
            q->GetEventOutlet()->Broadcast( cscmdQuit );
        }
        break;

        case CSKEY_UP:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camForward = true;
          }
          else
          {
            try
            {        
              if( !running )
              {
                running = true;
                act->stopAction( "stand", 0.0 );
                act->setActionCycle( "run", 1.0 );
              }
            }
            catch( std::exception &e )
            {
              LOG( "betaverse::SetupFrame()", 0, "Problem with something: " << e.what() );
            }
          }
        }
        break;
        
        case CSKEY_DOWN:
        {  
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camBackward = true;
          }
          else
          {
            try
            {         
              if( !runningBack )
              {
                runningBack = true;
                act->stopAction( "stand", 0.0 );   
                act->setActionCycle( "run", -1.0 );
              }
   
            }
            catch( std::exception &e )
            {
              LOG( "betaverse::SetupFrame()", 0, "Problem with something: " << e.what() );
            }
          }  
        }
        break;
        
        case CSKEY_RIGHT:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camRight = true;
          }
          else
          {
            turnRight = true;
          }
        }
        break;
        
        case CSKEY_LEFT:
        {  
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camLeft = true;
          }  
          else
          {
            turnLeft = true; 
          }
        }
        break;       
      
      
        case CSKEY_PGUP:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camUp = true;
          }
        }
        break;
        
        case CSKEY_PGDN:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camDown = true;
          }
        }
        break;
      }
    }
    break;
    
    case csKeyEventTypeUp:
    {
      switch( csKeyEventHelper::GetCookedCode( &Event ) )
      {
        case CSKEY_UP:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camForward = false;
          }
          else
          {
            try
            {        
              if( running )
              {
                running = false;
                act->stopAction( "run", 0.0 );
                act->setActionCycle( "stand", 1.0 );
              }
            }
            catch( std::exception &e )
            {
              LOG( "betaverse::SetupFrame()", 0, "Problem with something: " << e.what() );
            }
          }
        }
        break;
        
        case CSKEY_DOWN:
        {  
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camBackward = false;
          }
          else
          {
            try
            {          
              if( runningBack )
              {
                runningBack = false;
                act->stopAction( "run", 0.0 );   
                act->setActionCycle( "stand", 1.0 );
              }
            }
            catch( std::exception &e )
            {
              LOG( "betaverse::SetupFrame()", 0, "Problem with something: " << e.what() );
            }
          }  
        }
        break;
        
        case CSKEY_RIGHT:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camRight = false;
          }
          else
          {
            turnRight = false;
          }
        }
        break;
        
        case CSKEY_LEFT:
        {  
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camLeft = false;
          }  
          else
          {
            turnLeft = false; 
          }
        }
        break; 
        
        case CSKEY_PGUP:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camUp = false;
          }
        }
        break;
        
        case CSKEY_PGDN:
        {
          if( kbd->GetKeyState( CSKEY_CTRL ) )
          {
            camDown = false;
          }
        }
        break;
                       
      }
    }
    break;
  }
   
  return false;
}
        
/*            
    case csevMouseClick:
      switch( Event.Mouse.Button )
      {
        case 2:
          if( mouseLook )
            mouseLook = false;
          else
            mouseLook = true;

          bSink->Chat();
        break;
      }
    break;


    case csevMouseDown:
      switch( Event.Mouse.Button )
      {
        case 1:
          if( mouseLook )
          {
            c->Move( CS_VEC_FORWARD, speed );
          }
        break;
      }
    break;

    case csevMouseUp:
      switch( Event.Mouse.Button )
      {
        case 1:
          camMove = false;
        break;
      }
    break;

    case csevMouseMove:
      if( mouseLook )
      {
        camMove = false;
        int distX = Event.Mouse.x - centerX;
        int distY = Event.Mouse.y - centerY;
        //printf( "distX: %i, distY: %i\n", distX, distY );
        if( distY > distX )
        {
          if( distY > 25 )
          {
            printf( "Right: " );
            c->GetTransform ().RotateThis( CS_VEC_ROT_RIGHT, speed);
          }
          if( distY < -25 )
          {
            printf( "Left: " );
            c->GetTransform ().RotateThis( CS_VEC_ROT_LEFT, speed);
          }
        }
        else
        {
          if( distX > 25 )
          {
            printf( "Up: " );
            c->GetTransform ().RotateThis( CS_VEC_TILT_UP, speed);
          }
          if( distX < -25 )
          {
            printf( "Down" );
            c->GetTransform ().RotateThis ( CS_VEC_TILT_DOWN, speed);
          }
        }

        g2d->SetMousePosition( centerX, centerY );
      }
*/

void BetaVerse::Report( int severity, const char* msg, ... )
{
  va_list arg;
  va_start( arg, msg );
  csRef<iReporter> rep( CS_QUERY_REGISTRY( object_reg, iReporter ));
  if( rep )
    rep->ReportV( severity, "crystalspace.application.BetaVerse", msg, arg );
  else
  {
    csPrintfV( msg, arg );
    csPrintf( "\n" );
  }
  va_end( arg );
}

csPixmap * BetaVerse::LoadPixmap( char *file )
{
  csRef<iGraphics3D> myG3D = CS_QUERY_REGISTRY( object_reg, iGraphics3D );
  csRef<iTextureManager> txtmgr = myG3D->GetTextureManager();
  csRef<iDataBuffer> text_data = this->VFS->ReadFile( file );
  csRef<iImageIO> loader = CS_QUERY_REGISTRY( object_reg, iImageIO );
  csRef<iImage> ifile = loader->Load( text_data->GetUint8(),
    text_data->GetSize(), txtmgr->GetTextureFormat() );

  csRef<iTextureHandle> txt = txtmgr->RegisterTexture( ifile, CS_TEXTURE_2D );
  txt->Prepare();

  csPixmap *pixmap = new csSimplePixmap( txt );

  return pixmap;
}

void BetaVerse::Quit()
{
  csRef<iEventQueue> q = CS_QUERY_REGISTRY (object_reg, iEventQueue);
  if (q)
  {
    q->GetEventOutlet()->Broadcast (cscmdQuit);
  }
}

bool BetaVerse::LoadMap()
{
  // Set VFS current directory to the level we want to load.
  csRef<iVFS> VFS( CS_QUERY_REGISTRY( object_reg, iVFS ));
  VFS->ChDir( "/lib/beta/wrld" );
  // Load the level file which is called 'world'.
  if( !loader->LoadMapFile( "world" ))
  {
    csReport( object_reg, CS_REPORTER_SEVERITY_ERROR,
        "crystalspace.application.betaverse",
        "Couldn't load level!" );
    return false;
  }
  engine->Prepare();

  return true;
}

bool BetaVerse::ReadScript( iObjectRegistry *objreg, const char* script )
{
  // Start the 'script' which should be in the form of:
  // "/config/local.lst"

  csRef<iVFS> vfs( CS_QUERY_REGISTRY( objreg, iVFS ));
  if( !vfs )
  {
    printf( "No VFS for ReadScript!\n" );
    return false;
  }

  bParse = new BetaParser();

  if( !bParse->Parse( objreg, script ) )
  {
    csReport( objreg, CS_REPORTER_SEVERITY_ERROR,
      "crystalspace.application.betaverse",
      "Can't Init BetaParser" );
    return false;
  }
#ifdef BV_DEBUG
// A little test to see what's squashing our pdata struct on linux
  printf( "*** DEBUG ***\n" );
  printf( "*** pdata->name: %s\n", pdata.name );
  printf( "*** pdata->irc_srv: %s\n", pdata.irc_srv );
  printf( "*** DEBUG END ***\n" );
#endif

  return true;
}

void BetaVerse::StartAnim( const char *action, float wieght )
{
  if( player )
  {
    csRef<iSpriteCal3DState> cal3dstate(SCF_QUERY_INTERFACE(player->GetMeshObject(),
      iSpriteCal3DState));
    if( cal3dstate )
      cal3dstate->SetAnimCycle( action, wieght );
    else
      csReport( object_reg, CS_REPORTER_SEVERITY_ERROR,
        "crystalspace.application.betaverse",
        "Failed to Ref iSpriteCal3DState" );
  }
}


void BetaVerse::MakeColliders()
{
  iMeshWrapper *mesh;
  iMeshList *mL = engine->GetMeshes();
  for( int c = 0; c++; c <= mL->GetCount() )
  {
    mesh = mL->Get( c );
    //dynSys->AttachColliderMesh( mesh, mesh->GetMovable()->GetTransform(), 1.0, 0.0 );
  }
}

#define RegisterFactory( factoryclass )   \
    factory = new factoryclass( pawsMan );
    
void BetaVerse::DeclareBvFactories()
{
  pawsWidgetFactory* factory;

  RegisterFactory( ContactWindowFactory );
  RegisterFactory( ChatWindowFactory );
}

void BetaVerse::LoadPawsWidgets()
{
  pawsMan->LoadWidget( "/config/contact.xml" );
  pawsMan->LoadWidget( "/config/chat.xml" );
}
// End
