#ifndef __BETAVERSE_H__
#define __BETAVERSE_H__

#include <stdarg.h>
#include "csgeom/math2d.h"
#include "csgeom/math3d.h"
//#include "csutil/csstring.h"
#include "csutil/stringarray.h"
#include "vos/vos/site.hh"
#include "vos/metaobjects/a3dl/model.hh"

using namespace VOS;

struct iEngine;
struct iSector;
struct iView;
struct iFont;
struct iFile;
struct iImageLoader;
struct iLoaderPlugin;
struct iMeshWrapper;
struct iMeshFactoryWrapper;
struct iLoader;
struct iConsoleOutput;
struct iObjectRegistry;
struct iKeyboardDriver;
struct iVirtualClock;
struct iEvent;
struct iGraphics3D;
struct iGraphics2D;
struct iVFS;
struct iCamera;
struct csVector3;
struct csMouse;
struct iDynamics;
struct iDynamicSystem;
struct iRigidBody;
struct iMovable;
struct iTextureManager;
struct iVosA3DL;
//struct VOS::Vobject;

struct PawsManager;
struct pawsMainWidget;
struct ContactWindow;
struct ChatWindow;

//class csStringArray;
class csTransform;
class csPixmap;
class BetaNet;
class BetaParser;
class BetaSink;
//csString ircData;
struct ParserData;

class BetaVerse
{
public:
  iObjectRegistry* object_reg;
  BetaNet *bNet;
  BetaParser *bParse;
  csString ircData;
  csStringArray *roommates;
  csStringArray *buddies;
//  ParserData *pdata;
  PawsManager *pawsMan;
  pawsMainWidget *pawsMain;
  ContactWindow *contact;
  ChatWindow *chat;
  csRef<iVosA3DL> vosa3dl;
  vRef<VOS::Site> site;
  vRef<VOS::Site> remoteSite;
  vRef<A3DL::Model> playerModel;
   
  
private:
  csRef<iEngine> engine;
  csRef<iSector> room;
  csRef<iView> view;
  csRef<iGraphics3D> g3d;
  csRef<iGraphics2D> g2d;
  csRef<iVFS> VFS;
  csRef<iConsoleOutput> myConsole;
  csRef<iLoader> loader;
  csRef<iVirtualClock> vc;
  csRef<iKeyboardDriver> kbd;
  csRef<iFont> font;
  //csRef<iDynamics> dyn;
  //csRef<iDynamicSystem> dynSys;
  csRef<iMeshWrapper> player;
  csRef<iMeshFactoryWrapper> ballFact;
  //csRef<iRigidBody> pBody;

  
  int col_red, col_blue, col_white, col_black;
  int col_yellow, col_cyan, col_green, col_gray;
  int counter;
  int centerX, centerY;
  char message[255];
  csTicks message_timer;
  bool message_error;
  bool mouseLook;
  bool camRotate, camMove;
  bool playerRun;
  bool running, runningBack, turnRight, turnLeft;
  bool camForward, camBackward, camRight, camLeft, camUp, camDown;

  float rotY;
  float rotX;
  
  csPixmap *loading;

  csMouse *mouse;
  iCamera *c;
  csVector3 *camV;
  iTextureManager* txtmgr;

  csStringArray stateslist,actionlist,activelist,meshlist,morphanimationlist,socketlist;
 
public:
  BetaVerse();
  virtual ~BetaVerse();

  bool Initialize(int argc, const char* const argv[], const char *iConfigName);

  void Report (int severity, const char* msg, ...);

  void SetupFrame();
  void FinishFrame();
  bool HandleEvent (iEvent &Event);
  void Quit();

  bool ReadScript( iObjectRegistry *objreg, const char* script );
  bool WriteScript( iObjectRegistry *objreg, const char* script );



protected:
  csPixmap *LoadPixmap (char *file);
  bool LoadMap();
  iRigidBody* CreateSphere (void);
  
  void StartAnim( const char *action, float wieght );
  void MakeColliders();
  void DeclareBvFactories();
  void LoadPawsWidgets();
  //csPtr<iMeshWrapper> LoadMesh();
};

#endif // __Beta3D_H__
