//#include <std>
#include "cssysdef.h"
#include "chat.h"
#include "net.h"
#include "parser.h"
#include "betaverse.h"
#include "paws/pawsmanager.h"
#include "paws/pawslistbox.h"
#include "paws/pawsbutton.h"
#include "paws/pawstextbox.h"

#include <stdlib.h>
#include <stdio.h>
#include "vos/metaobjects/misc/avatar.hh"

extern ParserData pdata;

SCF_IMPLEMENT_IBASE( ChatWindow )
SCF_IMPLEMENT_IBASE_END

ChatWindow::ChatWindow( PawsManager *pawsMan) : 
  pawsWidget( pawsMan )
{
}

ChatWindow::~ChatWindow()
{
}

bool ChatWindow::PostSetup()
{
  chatlist = (pawsListBox *)FindWidget( "ChatList" );
  send = (pawsButton *)FindWidget( "Send" );
  input = (pawsEditTextBox *)FindWidget( "InputBox" );
  
  //input->SetMultiLine( true );
  //printf( "Contacts WM: %u\n", WindowManager() );
  return true;
}

void ChatWindow::Init( BetaVerse *beta )
{
  bv = beta;
}

bool ChatWindow::OnButtonPressed( int button, int keyModifier, pawsWidget* widget )
{
  //printf( "ChatWindow::OnButtonPressed() widget->GetName(): %s\n", widget->GetName() );
  csString name = widget->GetName();
  if( name == "Send" )
  {
    //printf( "ChatWindow::OnButtonPressed(): Send Button\n" );
    SendData();
    input->SetText( "" );
  }
    
  return true;
}

void ChatWindow::UpdateChat( csString from, csString msg )
{
  csString line, temp;
  size_t pos;

  if( msg.Length() > 35 )  
  {
    pos = msg.FindLast( ' ', 35 );
    if( pos == (size_t)-1 )
      pos = 35;
    msg.SubString( line, pos, msg.Length() - pos );
    msg.SubString( temp, 0, pos );
    LOG( "ChatWindow::UpdateChat()", 2, "line: " << line.GetData() << 
      " temp: " << temp.GetData() );
    msg = temp;   
  }
  pawsListBoxRow *row;
  pawsTextBox *nick, *text;
  
  //CS_ASSERT( contacts );
    
  row = chatlist->NewRow();
  CS_ASSERT( row );
  nick = (pawsTextBox *)row->GetColumn(0);
  if( nick == NULL )
    printf( "Error: Null Text Box\n" );
  nick->SetText( from.GetData() ); 
  
  LOG( "ChatWindow::UpdateChat()", 2, "msg: " << msg.GetData() );
  text = (pawsTextBox *)row->GetColumn(1);
  if( text == NULL )
    printf( "Error: Null Text Box\n" );
  text->SetText( msg.GetData() );  

  if( line.Length() != 0 )
    UpdateChat( " ", line );
  return;
}

void ChatWindow::SendData()
{
  csString msg;
  //msg.Append( "PRIVMSG " );
  //msg.Append( pdata.ChannelData.chan );
  //msg.Append( " :" );
  //UpdateChat( pdata.nick, input->GetText() );
  //LOG( "ChatWindow::SendData()", 1, "input->GetText(): " << input->GetText() );
  msg.Append( input->GetText() );
  
  vRef<Talkative> localTalk;
  vRef<Talkative> remoteTalk;
  vRef<Talkative> sectorTalk;

  try
  {
    localTalk = meta_cast<Talkative>( bv->site );
    remoteTalk = meta_cast<Talkative>( bv->playerModel );
    sectorTalk = meta_cast<Talkative>( bv->remoteSite->findObject( "world" ) );
  
    TalkMessage m = TalkMessage( localTalk, msg.GetData() );
  
    remoteTalk->resend( sectorTalk, m );
  }
  catch( std::exception &e )
  {
    LOG( "ChatWindow::SendData()", 1, "Could not send data: " << e.what() );
  }
    
  return;
}

ChatListener::ChatListener( ChatWindow *c )
{
  chat = c;
}

void ChatListener::notifyTalkMessage( VOS::TalkMessage &m )
{
  csString nick;
  csString data;
  
  vRef<VOS::Avatar> avatar = meta_cast<VOS::Avatar>( m.getOrigin() );
  try
  {
    nick.Append( avatar->getNick().c_str() );
  }
  catch( std::exception &e )
  {
    LOG( "ChatListener::notifyTalkMessage()", 1, "Problem with avatar: " << e.what() );
  }
  
  data.Append( m.getText().c_str() );
  
  chat->UpdateChat( nick, data );
}
