/*******************************************************************************
* 
* BetaNet.cpp
*  - written by: mason m4s0n_j0y3r5@yahoo.com
*
* This file contains the networking class for BetaVerse
*
*******************************************************************************/

#include "cssysdef.h"
#include "csutil/csstring.h"
#include "csutil/stringarray.h"
#include "csutil/scf.h"
#include "csutil/cfgfile.h"
#include "iengine/engine.h"
#include "ivaria/reporter.h"
#include "iutil/document.h"

#include "net.h"
#include "contact.h"
#include "chat.h"
#include "parser.h"
#include "betaverse.h"

#include "paws/pawsmanager.h"


extern ParserData *pData;

BetaNet::BetaNet()
{
}

bool BetaNet::Init( iObjectRegistry *objreg, csString server, int port, BetaVerse *beta )
{
  buffer = false;
  first = true;
  bv = beta;
  return 1;
}

int BetaNet::SendIRC( csString msg )
{
   
  return 1;
}

int BetaNet::RecvIRC( csString &msg )
{
 
  return 1;
}

void BetaNet::ParseLine( csString msg )
{
  printf( "ParseLine(): msg: %s\n", msg.GetData() );
  if (msg.GetData()[0] == ':')
  {
    /*
    *  Message in the form:
    *  message  =  [ ":" prefix SPACE ] command [ params ] crlf
    *  prefix    =  servername / ( nickname [ [ "!" user ] "@" host ] )
    *  command   =  1*letter / 3digit
    *  params    =  *14( SPACE middle ) [ SPACE ":" trailing ]
    *            =/ 14( SPACE middle ) [ SPACE [ ":" ] trailing ]
    */

    // Find prefix
    size_t pos1 = msg.FindFirst(' ', 0);
    csString from;
    msg.SubString( from, 1, pos1-1 );

    pos1++;
    // Find command
    size_t pos2 = msg.FindFirst(' ', pos1 );
    csString command;
    msg.SubString( command, pos1, pos2 - pos1);

    pos2++;
    // Check whether there is any params
    size_t pos3 = msg.FindFirst( ':', pos2 );
    csString param;

    if ((pos3 - 1) != pos2) // We have params
    {
      msg.SubString(param, pos2, pos3 - pos2);
    }

    pos3++;
    // Get rest (whats after the ':', if there were any ':')
    csString rest;
    if (pos3 != msg.Length() )
    {
      msg.SubString( rest, pos3, msg.Length() - pos3 );
    }
    printf( "From: %s\n", from.GetData() );
    printf( "Command: %s\n", command.GetData() );
    printf( "Param: %s\n", param.GetData() );
    printf( "Rest: %s\n", rest.GetData() );
    if( command == "PRIVMSG" )
      PrivMsg( from, command, param, rest );
    else if( command == "353" )
      Names( from, command, param, rest );
    else if( command == "JOIN" )
      Join( from, command, param, rest );
    else if( command == "PART" )
      Part( from, command, param, rest );
      

  }
  else
  {
    // Parse csString in form, eg. 'PING :23523525'
    size_t pos1 = msg.FindFirst(' ', 0);
    csString command;
    msg.SubString(command, 0, pos1);

    // Check whether there are any params
    size_t pos2 = msg.FindFirst(':', pos1);
    csString param;
    
    pos1++;

    if ((pos2 - 1) > pos1)
    {
      // We have params
      msg.SubString( param, pos1, pos2 - pos1 );
    }

    pos2++;

    // Get rest (whats after the ':')
    csString rest;
    msg.SubString(rest, pos2, msg.Length() - pos2 );

    // Redirect to the right parsing function...
    if (command == "PING")
      Ping(rest);
  }
  // Clear the string... it is not needed anymore
  msg.Clear();
}

void BetaNet::Parse()
{
  csString data;
  csString line;
  size_t len, start;
  len = start = 0;

  int bytes = RecvIRC(data);


  if( bytes == -1 )
    return;
  else {
    //printf( "Parse() bytes: %i\n", bytes );

    int c, pos;
    for (c = 0, pos = 0; pos < data.Length(); pos++)
      if (data.GetAt(pos) == '\n')
	c++;

    //printf("Parse() newlines: %i\n", c);
  }

  if( buffer )
  {
    //printf("Parse() buffer length: %i\n", buf.Length());
    buf.Append( data );
    data = buf;
    //printf( "Parse() buf: %s\n", buf.GetData() );
    //printf( "Parse() data: %s\n", data.GetData() );
    buf.Clear();
    buffer = false;
  }
  //printf( "Parse() data.Length(): %i\n", data.Length() );

  do
  {
    start = len;
    len = data.FindFirst( '\n', start );
    //if( len != (size_t)-1 )
    //  len--;
    //printf( "Parse() len: %i\n", len );
    if( len == (size_t)-1 )
    {
      buf.Append(data.GetData() + start);
      //printf( "Parse() buffer (%i-%i-%i): %s\n", start, data.Length(),                  buf.Length(), buf.GetData() );
      buffer = true;
      break;
    }
    else
    {
      data.SubString( line, start, len - start);
      if( line.Length() == 0 )
        return;
      //printf( "Parse() line: %s\n", line.GetData() );
      ParseLine( line );
      len++;
    }

  }while( len <= data.Length() );
  //clear the data string
  data.Clear();
}

void BetaNet::Ping( csString msg )
{
  printf( "Ping()\n" );
  csString line = "PONG :";
  line.Append( msg );

  SendIRC( line );
  return;
}

void BetaNet::PrivMsg( csString from, csString command, csString params, csString rest )
{
  size_t start, len;
  start = len = 0;
  
  rest.Append( '\0' );
  len = from.FindFirst( '!', start );
  csString nick;
  from.SubString( nick, start, len - start );
  
  bv->chat->UpdateChat( nick, rest );
}

void BetaNet::Names( csString from, csString command, csString params, csString rest )
{
  size_t start, len;
  start = len = 0;
  
  int count = 0;
  
  csString name;

  do
  {
    start = len;
    len = rest.FindFirst( ' ', start );
    printf( "Names() start: %i len: %i\n", start, len );
    if( len == (size_t) -1 )
      break;
    rest.SubString( name, start, len - start );
    printf( "Names() name: %s\n", name.GetData() );
    
    bv->contact->AddRoommate( name );
    //printf( "Names() room[%i]: %s\n", count, bv->roommates->Get(count) );
    
    //bSQL->GetDBinfo( name.GetData() );
    len++;
    count++;
  }while( len <= rest.Length() );
  
  bv->contact->ShowRoommates();
}

void BetaNet::Join( csString from, csString command, csString params, csString rest )
{
  if( first )
  { 
    first = false;
    return;
  }
  size_t start, len;
  start = len = 0;
  
  rest.Append( '\0' );
  len = from.FindFirst( '!', start );
  csString nick;
  from.SubString( nick, start, len - start );
  
  bv->contact->AddRoommate( nick );
}

void BetaNet::Part( csString from, csString command, csString params, csString rest )
{
  size_t start, len;
  start = len = 0;
  
  rest.Append( '\0' );
  len = from.FindFirst( '!', start );
  csString nick;
  from.SubString( nick, start, len - start );
  
  bv->contact->RemoveRoommate( nick );
}
