/*
    Copyright (C) 2003 by Wyatt Miler

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cssysdef.h"
//#include "cssys/sysfunc.h"
#include "csutil/csevent.h"
#include "cstool/initapp.h"
#include "csutil/cmdhelp.h"
#include "csutil/csinput.h"
#include "ivideo/natwin.h"
#include "ivaria/conout.h"
#include "iengine/engine.h"
#include "imap/reader.h"
#include "igraphic/imageio.h"
#include "imap/parser.h"
#include "ivaria/reporter.h"
#include "iutil/eventq.h"
#include "iutil/virtclk.h"
#include "iutil/plugin.h"
#include "iutil/vfs.h"
#include "qsqrt.h"
#include "csutil/scanstr.h"
#include "iutil/databuff.h"
#include "csutil/xmltiny.h"
#include "qsqrt.h"

#include "parser.h"
#include "betaverse.h"
//#include "betasink.h"
#include "net.h"
#include <stdio.h>
//#include "inetwork/sockerr.h"

#define  QUERY_REG(myPlug, iFace, errMsg) \
  myPlug = CS_QUERY_REGISTRY (BetaAWS->object_reg, iFace); \
  if (!myPlug) \
  { \
    BetaAWS->Report (CS_REPORTER_SEVERITY_ERROR, errMsg); \
    return false; \
  }

#define STRDUP(str) \
({ \
  char * n = new char [strlen((str)) + 1]; \
  strcpy(n, (str)); \
})

const char* budlist;
ParserData pdata;

// Define all tokens used through this file
enum
{
  XMLTOKEN_ATTACH = 1,
  XMLTOKEN_NAME,
  XMLTOKEN_TYPE,
  XMLTOKEN_NICK,
  XMLTOKEN_IRCSERVER,
  XMLTOKEN_USER,
  XMLTOKEN_CHANNEL,
  XMLTOKEN_PASSWD,
  XMLTOKEN_CACHE_SIZE,
  XMLTOKEN_CACHE_TIME,
  XMLTOKEN_LOCATION,
  XMLTOKEN_SUBTYPE,
  XMLTOKEN_LOC_LINK,
  XMLTOKEN_MAP,
  XMLTOKEN_BUDLIST,
  XMLTOKEN_FILE
};

// \Convenince function to grap things out of pdata.
const char *
BetaParser::PdataGrap(const char* key)
{
  return NULL;
};

// \Workhourse routine for the parser reader.
bool
BetaParser::Parse(iObjectRegistry *objreg, const char* filename)
{
  xmltokens.Register ("attach", XMLTOKEN_ATTACH);
  xmltokens.Register ("name", XMLTOKEN_NAME);
  xmltokens.Register ("type", XMLTOKEN_TYPE);
  xmltokens.Register ("nick", XMLTOKEN_NICK);
  xmltokens.Register ("ircserver", XMLTOKEN_IRCSERVER);
  xmltokens.Register ("user", XMLTOKEN_USER);
  xmltokens.Register ("channel", XMLTOKEN_CHANNEL);
  xmltokens.Register ("passwd", XMLTOKEN_PASSWD);
  xmltokens.Register ("cache-size", XMLTOKEN_CACHE_SIZE);
  xmltokens.Register ("cache-time", XMLTOKEN_CACHE_TIME);
  xmltokens.Register ("location", XMLTOKEN_LOCATION);
  xmltokens.Register ("subtype", XMLTOKEN_SUBTYPE);
  xmltokens.Register ("loc-link", XMLTOKEN_LOC_LINK);
  xmltokens.Register ("map", XMLTOKEN_MAP);
  xmltokens.Register ("budlist", XMLTOKEN_BUDLIST);
  xmltokens.Register ("file", XMLTOKEN_FILE);

  csRef<iVFS> vfs (CS_QUERY_REGISTRY (objreg, iVFS));
  csRef<iDataBuffer> buf (vfs->ReadFile (filename));

  if (!buf || !buf->GetSize ())
  {
    csReport (objreg,CS_REPORTER_SEVERITY_WARNING,
    	"Could not open local config file '%s' on VFS! BetaParser:Parse()", filename);
    return 0;
  }

  csRef<iDocumentSystem> xml (
  	CS_QUERY_REGISTRY (objreg, iDocumentSystem));
  if (!xml) xml = csPtr<iDocumentSystem> (new csTinyDocumentSystem ());
  csRef<iDocument> doc = xml->CreateDocument ();
  const char* error = doc->Parse (buf);
  if (error != NULL)
  {
    csReport (objreg, CS_REPORTER_SEVERITY_ERROR,
    	"XML error '%s' for file '%s' while parsing!", error, filename);
    exit (0);
  }
  LoadMain(doc->GetRoot());
  return 1;
};

// \Error check before calling LoadLst()
void BetaParser::LoadMain (iDocumentNode* node)
{
  csRef<iDocumentNode> child = node->GetNode ("file");
  if (!child)
  {
    printf("LoadMain() Expected '<file>'!\n");
    exit (-1);
  }
  LoadLst (child);
};

// \Workhourse routine that reads the local.lst file an loads the ParserData struct.
void BetaParser::LoadLst (iDocumentNode* node)
{
// Just because Jorrit said so.
  pdata.name			= NULL;
  pdata.nick			= NULL;
  pdata.irc_srv			= NULL;
  pdata.passwd			= NULL;
  pdata.cache_size		= NULL;
  pdata.cache_time		= NULL;
  pdata.usr			= NULL;
  pdata.ChannelData.chan	= NULL;
  pdata.ChannelData.next	= NULL;
  pdata.LocationData.loc	= NULL;
  pdata.LocationData.loc_link	= NULL;
  pdata.LocationData.next	= NULL;
  pdata.BudData.buddy		= NULL;
  pdata.BudData.next		= NULL;

  csRef<iDocumentNodeIterator> it = node->GetNodes ();

  while (it->HasNext ())
  {
    csRef<iDocumentNode> child = it->Next ();
    const char* value = child->GetValue ();
    csStringID id = xmltokens.Request (value);

    switch (id)
    {
      case XMLTOKEN_ATTACH:
      {
        break;
      }
      case XMLTOKEN_NAME:
      {
         pdata.name = STRDUP(child->GetContentsValue ());
         printf("Loaded NAME token with... %s\n", pdata.name);
         break;
      }
      case XMLTOKEN_NICK:
      {
         pdata.nick = STRDUP(child->GetContentsValue ());
         printf("Loaded NICK token with... %s\n", pdata.nick);
         break;
      }
      case XMLTOKEN_IRCSERVER:
      {
         pdata.irc_srv = STRDUP(child->GetContentsValue ());
         printf("Loaded IRCSERVER token with... %s\n", pdata.irc_srv);
         break;
      }
      case XMLTOKEN_USER:
      {
         pdata.usr = STRDUP(child->GetContentsValue ());
         printf("Loaded USER.USR token with... %s\n", pdata.usr);
         break;
      }
      case XMLTOKEN_CHANNEL:
      {
         pdata.ChannelData.chan = STRDUP(child->GetContentsValue ());
         printf("Loaded CHANNEL.CHAN token with... %s\n", pdata.ChannelData.chan);
         break;
      }
      case XMLTOKEN_PASSWD:
      {
         pdata.passwd = STRDUP(child->GetContentsValue ());
         printf("Loaded PASSWD token with... %s\n", pdata.passwd);
         break;
      }
      case XMLTOKEN_CACHE_SIZE:
      {
         pdata.cache_size = STRDUP(child->GetContentsValue ());
         printf("Loaded CACHE_SIZE token with... %s\n", pdata.cache_size);
         break;
      }
      case XMLTOKEN_CACHE_TIME:
      {
         pdata.cache_time = STRDUP(child->GetContentsValue ());
         printf("Loaded CACHE_TIME token with... %s\n", pdata.cache_time);
         break;
      }
      case XMLTOKEN_LOCATION:
      {
         pdata.LocationData.loc = STRDUP(child->GetContentsValue ());
         printf("Loaded LOCATION.LOC token with... %s\n", pdata.LocationData.loc);
         break;
      }
      case XMLTOKEN_LOC_LINK:
      {
         pdata.LocationData.loc_link = STRDUP(child->GetContentsValue ());
         printf("Loaded LOCATION.LOC_LINK token with... %s\n", pdata.LocationData.loc_link);
         break;
      }
      case XMLTOKEN_MAP:
      {
         pdata.MapData.map = STRDUP(child->GetContentsValue ());
         printf("Loaded MAP.MAP token with... %s\n", pdata.MapData.map);
         break;
      }
      case XMLTOKEN_BUDLIST:
      {
         pdata.BudData.buddy = STRDUP(child->GetContentsValue ());
         printf("Loaded BUDLIST.BUDDY token with... %s\n", pdata.BudData.buddy);
         break;
      }
      case XMLTOKEN_FILE:
      {
//         printf("Loaded 'FILE' Found %s\n", child->GetValue ());
         break;
      }
      case XMLTOKEN_SUBTYPE:
      case XMLTOKEN_TYPE:
      {
//        printf("Loaded 'TYPE' Found '%s'\n", child->GetValue ());
        break;
      }
      default:
	  printf("Unexpected token '%s'!", value);
	exit (0);
    }
  }
// WHM
    printf("Successfully loaded config file!\n");

};

// \Part of the saving routines, puts the local.lst together before saving out.
bool
BetaParser::Write(iObjectRegistry *objreg, const char* filename)
{
  const char* mydefault = "/this/config/default.lst";
  const char* local = "/this/config/local.lst";

  printf("BetaParser::Write() starting with file: %s\n", filename);
  csRef<iVFS> vfs (CS_QUERY_REGISTRY (objreg, iVFS));
  csRef<iDataBuffer> buf (vfs->ReadFile (filename));

  if (!buf || !buf->GetSize ())
  {
    csReport (objreg,CS_REPORTER_SEVERITY_WARNING,
    	"Could not open local config file '%s' on VFS. Creating new one from default.lst BetaParser:Write()", filename);

    // No local.lst, so we make a copy of default.lst.
    csRef<iDataBuffer> buf (vfs->ReadFile (mydefault));
    if (1)
    {
      char* newname = new char [strlen (local)];
      strcpy (newname, local);
      vfs->WriteFile (newname, **buf, buf->GetSize ());
      csReport (objreg,CS_REPORTER_SEVERITY_WARNING,
    	"Wrote to local config file '%s'. BetaParser:Write()", newname);
      delete[] newname;
    }

  } else {
    csReport (objreg,CS_REPORTER_SEVERITY_WARNING,
    	"Found local config file '%s' on VFS. Copying new data to file. BetaParser:Write()", filename);
  // WHM Not implemented yet!
  // WHM Note to self, look how cs/apps/tools/cs2xml does these things
  // WHM Further note to self, we have to implement our own ordering routine here
  // WHM as cs2xml uses the CS parser stuffs
  }

/* WHM 
  csParser* parser = new csParser (true);
  parser->ResetParserLine ();
     
  CS_TOKEN_TABLE_START (tokens)
  CS_TOKEN_TABLE_END
        
  char *data = **buf;
  char *name, *params;
  long cmd;
            
  if ((cmd = parser->GetObject (&data, tokens, &name, &params))
        != CS_PARSERR_EOF)
  {
    if (params)
    {
      char* tokname = ToLower (parser->GetUnknownToken (), true);
      csRef<iDocumentSystem> xml (csPtr<iDocumentSystem> (
        new csTinyDocumentSystem ()));
      csRef<iDocument> doc = xml->CreateDocument ();
      csRef<iDocumentNode> root = doc->CreateRoot ();
      csRef<iDocumentNode> parent = root->CreateNodeBefore (
            CS_NODE_ELEMENT, 0);
      parent->SetValue (tokname);
      ParseGeneral (tokname, parser, parent, params);
   
      doc->Write (vfs, vfspath);
      delete[] tokname;
    }
  }
        
  delete parser;
*/
  return true;
};

// \Error check before calling SaveLst()
void BetaParser::SaveMain (iFile *fp, iDocumentNode* node)
{
  csRef<iDocumentNode> child = node->GetNode ("file");
  if (!child)
  {
    printf("SaveMain() Expected '<file>'!\n");
    exit (-1);
  }
  SaveLst (fp, child);
};

// \Saves an online edited local.lst back out to disk
void BetaParser::SaveLst (iFile *fp, iDocumentNode* node)
{
  csRef<iDocumentNodeIterator> it = node->GetNodes ();

  while (it->HasNext ())
  {
    csRef<iDocumentNode> child = it->Next ();
    const char* value = child->GetValue ();
    csStringID id = xmltokens.Request (value);

    switch (id)
    {
      case XMLTOKEN_ATTACH:
      {
        printf ("Error! BetaParser:SaveLst()");
        break;
      }
    }
  }
};

// \Saves out a default local.lst file
void BetaParser::MakeDefaultLst (iDocumentNode* node)
{

};

BetaParser::BetaParser()
{
};

BetaParser::~BetaParser()
{
};

ParserData::~ParserData() {
  // NOTE: This destructor does not delete chained structures

  delete name;
  delete nick;
  delete irc_srv;

  delete passwd;
  delete cache_size;
  delete cache_time;
 
  delete LocationData.loc;
  delete LocationData.loc_link;
  delete LocationData.map;

  delete MapData.map;

  delete BudData.buddy;

  delete next; // chains on to the next ParserData destructor
};

// End


