#ifndef __BetaParser_H__
#define __BetaParser_H__

#include <stdarg.h>
#include "csutil/csstring.h"
#include "csutil/strhash.h"

struct iEngine;
struct iSector;
struct iView;
struct iFile;
struct iImageLoader;
struct iLoaderPlugin;
struct iLoader;
struct iConsoleOutput;
struct iObjectRegistry;
struct iEvent;
struct iVFS;
class csString;
class BetaNet;
class BetaAWS;

class BetaParser
{
public:
  iObjectRegistry* object_reg;

private:
  csRef<iEngine> engine;
  csRef<iVFS> VFS;
  csRef<iConsoleOutput> myConsole;
  csRef<iLoader> loader;
  csStringHash xmltokens;

  int counter;
  char message[255];

public:
  BetaParser();
  virtual ~BetaParser();

  bool Initialize(int argc, const char* const argv[], const char *iConfigName);

  bool HandleEvent (iEvent &Event);
  const char* PdataGrap (const char* name);
  bool Parse( iObjectRegistry *objreg, const char* filename );
  void LoadMain(iDocumentNode* node);
  void LoadLst(iDocumentNode* node);
  bool Write( iObjectRegistry *objreg, const char* filename );
  void SaveMain(iFile *fp, iDocumentNode* node);
  void SaveLst(iFile *fp, iDocumentNode* node);
  void MakeDefaultLst(iDocumentNode* node);

protected:
  bool LoadMap();

};

  /// \external Linked list of parser data
struct ParserData
{
  /** Strings are now copied, so they must be deleted at destruction. */
  ~ParserData();

    // Pointer to next file within the local.lst
    ParserData *next;
    void *key;
    iBase* data;

    // Name of this file within the local.lst
    const char     *name;
    // Nickname
    const char     *nick;
    // Irc server name
    const char     *irc_srv;
    // The user (not nick)
    const char 	   *usr;
    // Channel(s) this client can connect to
      struct ChannelData
      {
        const char  *chan;
        ChannelData *next;
      } ChannelData;
    // Password(s)
    const char      *passwd;
    // Cache-size in kB
    const char      *cache_size;
    // Seconds from epoch since cache last sync'd
    const char      *cache_time;
    // Location(s)
    // Where location connections link to: .loc_link and .map
      struct LocationData
      {
        const char     *loc;
        // Where location connections link to and the map it's in
        const char     *loc_link;
        const char     *map;
        LocationData   *next;
      } LocationData;
    // Maps
      struct MapData
      {
        const char  *map;
        MapData     *next;
      } MapData;
    // Buddylist
      struct BudData
      {
        const char  *buddy;
        BudData     *next;
      } BudData;
};

#endif // __BetaParser_H__
