/*
 * pawscheckbox.cpp
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#include <cssysdef.h>

#include "pawscheckbox.h"
#include "pawsmanager.h"
#include "pawstexturemanager.h"
#include "pawstextbox.h"

#include "util/log.h"
//---------------------------------------------------------------------------------
pawsCheckBox::pawsCheckBox( PawsManager* manager )
                :pawsButton( manager )
{
    down=false;
}

void pawsCheckBox::SetState( bool state )
{    
    checkBox->SetState( state );
}

pawsCheckBox::~pawsCheckBox()
{
}

bool pawsCheckBox::Setup( iDocumentNode* node )
{
    csRef<iDocumentNode> textNode = node->GetNode( "text" );        
    
    if ( !textNode )
    {
        Error2("%s XML is defined incorrectly. No <label /> tag found", name.GetData());
        return false;
    }
    
    csString pos(textNode->GetAttributeValue("position"));

    
    ///////////////////////////////////////////////////////////////////////
    // Create the check box
    ///////////////////////////////////////////////////////////////////////
    checkBox = (pawsButton*)WindowManager()->CreateWidget("pawsButton");
    AddChild( checkBox );

    csRect boxRect;
    csRect textRect;
    
    if ( pos == "left" )    
    {
        boxRect = csRect(  defaultFrame.Width()-16, 4, 
                              defaultFrame.Width(), 20 );
                              
        textRect   = csRect(  0, 4, defaultFrame.Width() - 16, defaultFrame.Height() );
    }
    
    if ( pos == "right" )    
    {
        boxRect = csRect(  4, 4, 
                              20, 20 );
                              
        textRect   = csRect(  22, 4, defaultFrame.Width()-16, defaultFrame.Height() );
    }
                
        
    checkBox->SetRelativeFrame( boxRect.xmin, boxRect.ymin, 
                                boxRect.Width(), boxRect.Height() );
    checkBox->SetUpImage( "radiooff" );
    checkBox->SetDownImage( "radioon" );
    checkBox->SetState( false );
    checkBox->PostSetup();
    checkBox->SetID( id );
     
    
    ///////////////////////////////////////////////////////////////////////
    // Create the textbox that has the current selected choice
    ///////////////////////////////////////////////////////////////////////    
    csString str(textNode->GetAttributeValue("string"));
    
    text = (pawsTextBox*)WindowManager()->CreateWidget("pawsTextBox");
    AddChild( text );

    // Puts the box at the edge of the text box widget
    text->SetRelativeFrame( textRect.xmin, textRect.ymin,
                            textRect.Width(), textRect.Height() );
    text->PostSetup();
    
    text->SetText(str);
    text->SetID( id );

    return true;               
}


bool pawsCheckBox::OnButtonPressed( int mouseButton, int keyModifier, pawsWidget* widget )
{
    if (checkBox)
        checkBox->SetState(!checkBox->GetState());

    if ( parent ) 
        return parent->OnButtonPressed( mouseButton, keyModifier, this );

    return false;
}

bool pawsCheckBox::GetState()
{
    if (checkBox != NULL)
        return checkBox->GetState();
    else
        return false;
}
