/*
 * pawscombobox.cpp - Author: Andrew Craig
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <cssysdef.h>
#include "pawscombo.h"
#include "pawsbutton.h"
#include "pawstextbox.h"
#include "pawslistbox.h"
#include "pawsmanager.h"

#define SHOW_LIST 100

pawsComboBox::pawsComboBox( PawsManager* manager )
             :pawsWidget( manager )
{
    oldHeight = 0;
    closed = true;  
    
    rows = 0;
    rowHeight = 0;
}

pawsComboBox::~pawsComboBox()
{

}

bool pawsComboBox::Setup( iDocumentNode* node ) 
{ 
    csRef<iDocumentNode> listNode = node->GetNode( "listbox" );  
    
    rows      = listNode->GetAttributeValueAsInt("rows");
    rowHeight = listNode->GetAttributeValueAsInt("height");
    text      = listNode->GetAttributeValue("text");
    
    return true;
}

bool pawsComboBox::PostSetup() 
{
    bool ok = false;

    ///////////////////////////////////////////////////////////////////////
    // Create the drop arrow button that will cause the list box to drop.
    ///////////////////////////////////////////////////////////////////////
    arrow = (pawsButton*)WindowManager()->CreateWidget("pawsButton");
    AddChild( arrow );

    // Puts the button at the edge of the text box widget
    arrow->SetRelativeFrame( defaultFrame.Width()-16, 4, 16, 16 );
    arrow->SetUpImage( "Up Arrow" );
    arrow->SetDownImage( "Up Arrow" );
    arrow->SetID( SHOW_LIST );
    
    ok = arrow->PostSetup();    
    
    
    ///////////////////////////////////////////////////////////////////////
    // Create the textbox that has the current selected choice
    ///////////////////////////////////////////////////////////////////////    
    itemChoice = (pawsTextBox*)WindowManager()->CreateWidget("pawsTextBox");
    AddChild( itemChoice );

    // Puts the button at the edge of the text box widget
    itemChoice->SetRelativeFrame( 0 , 4, defaultFrame.Width()-16, defaultFrame.Height() );
    ok = ok && itemChoice->PostSetup();

    itemChoice->SetText(text);
    itemChoice->SetID( SHOW_LIST );
    
    
    ///////////////////////////////////////////////////////////////////////
    // Create the drop down list box
    ///////////////////////////////////////////////////////////////////////      
    listChoice = (pawsListBox*)WindowManager()->CreateWidget("pawsListBox");
    AddChild( listChoice );
    listChoice->SetRelativeFrame( 0 , defaultFrame.Height(), defaultFrame.Width(), rows*rowHeight+5 );
    ok = ok && listChoice->PostSetup();
    listChoice->Hide();   
    listChoice->UseTitleRow( false ); 
    listChoice->SetID( id );
    
    csString widgetDef("<widget name=\"Text\" factory=\"pawsTextBox\" ></widget>");
    listChoice->SetTotalColumns( 1 );
    listChoice->SetColumnDef( 0, 
                              defaultFrame.Width()-32 , rowHeight,
                              widgetDef );

    return ok;
}

void pawsComboBox::NewOption(const csString & text)
{
    pawsListBoxRow * row;
    pawsTextBox * cell;
    
    row = NewOption();
    cell = dynamic_cast <pawsTextBox*> (row->GetColumn(0));
    if (cell != NULL)
        cell->SetText(text);
}

bool pawsComboBox::OnButtonPressed( int mouseButton, int keyModifier, pawsWidget* widget )
{
    switch ( widget->GetID() )
    {
        case SHOW_LIST:
        {
            if ( closed )
            {
                oldHeight = ScreenFrame().Height();                        
                SetSize( ScreenFrame().Width(), defaultFrame.Height()+rows*rowHeight );
                listChoice->Show();
                closed = false;
            }
            else
            {
                SetSize( ScreenFrame().Width(), oldHeight );
                listChoice->Hide();
                closed = true;
            }
            return true;
        }
    }    
    
    return false;
}

csString pawsComboBox::GetSelectedRowString()
{
    pawsListBoxRow* row = listChoice->GetSelectedRow();
    if (row == NULL) return "";
    
    pawsTextBox* thing = dynamic_cast <pawsTextBox*> (row->GetColumn(0));
    if (thing == NULL) return "";
    
    return thing->GetText();
}

void pawsComboBox::OnListAction( pawsListBox* widget, int status )
{
    itemChoice->SetText(GetSelectedRowString());
    
    if (!closed)
    {
        SetSize( ScreenFrame().Width(), oldHeight );
        listChoice->Hide();
        closed = true;
    }
    
    parent->OnListAction( widget, status );
}

int pawsComboBox::GetSelectedRowNum()
{
    return listChoice->GetSelectedRowNum();
}

void pawsComboBox::Select(int optionNum)
{
    pawsListBoxRow * row = listChoice->GetRow(optionNum);
    
    if (row != NULL)
        listChoice->Select(row);
}

pawsListBoxRow* pawsComboBox::NewOption()
{
    return listChoice->NewRow();
}
