/*
 * pawsmouse.cpp - Author: Andrew Craig
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
// pawsmouse.cpp: implementation of the pawsMouse class.
//
//////////////////////////////////////////////////////////////////////
#include <cssysdef.h>

#include <csutil/ref.h>
#include <ivideo/graph3d.h>

#include "pawsmouse.h"
#include "pawsmanager.h"
#include "pawstexturemanager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

pawsMouse::pawsMouse( PawsManager* manager )
{
    windowManager = manager;
    graphics3D = manager->GetGraphics3D();


    currentPosition = csPoint( 0,0 );
    deltas = csPoint( 0, 0 );
    cursorImage = new pawsImage( manager->GetObjectRegistry() );
}

pawsMouse::~pawsMouse()
{
    delete cursorImage;
}

void pawsMouse::SetPosition( int x, int y )
{
    deltas.x = x - currentPosition.x;
    deltas.y = y - currentPosition.y;

    currentPosition.x = x;
    currentPosition.y = y;
}

void pawsMouse::ChangeImage( const char* imageName )
{
    psImageDescription* imageDesc = windowManager->GetTextureManager()->GetImageDescription( imageName );   
    
    CS_ASSERT( imageDesc );
       
    cursorImage->Description( imageDesc );
}

void pawsMouse::ChangeImage( psImageDescription* desc )
{
    cursorImage->Description( desc );
}


void pawsMouse::Draw()
{    
    cursorImage->Draw( currentPosition.x , currentPosition.y );
}



