/*
 * pawsnumberpromptwindow.cpp - Author: Ondrej Hurt
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */


// CS INCLUDES
#include <cssysdef.h>

// COMMON INCLUDES
#include "util/log.h"

// PAWS INCLUDES
#include "pawsnumberpromptwindow.h"
#include "pawstextbox.h"
#include "pawsbutton.h"
#include "pawscrollbar.h"
#include "pawsmanager.h"
#include "pawsmainwidget.h"

class pawsNumberInput : public pawsWidget
{
public:
    pawsNumberInput(PawsManager* windowManager)
        : pawsWidget(windowManager)
    {
        SetRelativeFrameSize(110, 50);

        editBox = new pawsEditTextBox(windowManager);
        AddChild(editBox);
        editBox->SetRelativeFrame(20, 0, 70, 20);
        editBox->UseBorder("line");
        
        scrollBar = new pawsScrollBar(windowManager);
        AddChild(scrollBar);
        scrollBar->SetRelativeFrame(0, 30, 110, 20);
        scrollBar->SetHorizontal(true);
        scrollBar->SetTickValue(1);
        scrollBar->PostSetup();
        scrollBar->UseBorder("line");
    }

    pawsEditTextBox * editBox;
    pawsScrollBar * scrollBar;
};

pawsNumberPromptWindow::pawsNumberPromptWindow(PawsManager* manager) : pawsPromptWindow(manager)
{
    editBox        = NULL;
    scrollBar      = NULL;
    action         = NULL;

    SetMaxNumber(0);
    lastValidText = "";
}

bool pawsNumberPromptWindow::PostSetup()
{
    pawsPromptWindow::PostSetup();
    
    pawsNumberInput * numberInput = new pawsNumberInput(windowManager);
    AddChild(numberInput);
    inputWidget = numberInput;
    editBox    = numberInput->editBox;
    scrollBar  = numberInput->scrollBar;

    //LayoutWindow();

    return true;
}

bool pawsNumberPromptWindow::OnButtonPressed( int mouseButton, int keyModifier, pawsWidget* widget )
{
    if (action == NULL)
        return false;

    if (widget == okButton)
    {
        if (TextIsValidForOutput(editBox->GetText()))
            NumberWasEntered(atoi(editBox->GetText()));
        return true;
    }
    else if (widget == cancelButton)
    {
        if (action != NULL)
        {
            action->Execute(-1);
            delete action;
            action = NULL;
        }
        parent->DeleteChild(this);
        return true;
    }
    return false;
}

void pawsNumberPromptWindow::SetMaxNumber(int maxNumber)
{
    csString numberStr;
    int div;
    
    this->maxNumber = maxNumber;

    if (scrollBar != NULL)
        scrollBar->SetMaxValue(maxNumber);

    if (label != NULL)
    {
        numberStr = maxNumber;
        label->SetText("max = "+numberStr);
    }
    
    maxDigits = 0;
    div = maxNumber;
    while (div > 0)
    {
        maxDigits++;
        div /= 10;
    }
}

bool pawsNumberPromptWindow::OnScroll( int scrollDirection, pawsScrollBar* widget )
{
    lastValidText = scrollBar->GetCurrentValue();
    editBox->SetText(lastValidText);
    return true;
}

bool pawsNumberPromptWindow::TextIsValidForEditing(const csString & text)
{
    if (text.Length() > (unsigned int)maxDigits)
        return false;

    for (unsigned int i=0; i < text.Length(); i++)
        if ((text.GetAt(i) < '0') || (text.GetAt(i) > '9'))
            return false;

    if ( 
          (text.GetData() != NULL) 
             && 
          (atoi(text.GetData()) > maxNumber) 
       )
          return false;
    return true;
}

bool pawsNumberPromptWindow::TextIsValidForOutput(const csString & text)
{
    if (text.Length() == 0)
        return false;

    int number = atoi(text.GetData());
    return TextIsValidForEditing(text) && (number > 0) && (number >= minNumber);
}

bool pawsNumberPromptWindow::OnChange(pawsWidget * widget)
{
    csString text;
    
    if (widget == editBox)
    {
        text = editBox->GetText();
        if (TextIsValidForEditing(text))
        {
            lastValidText = text;
            if (TextIsValidForOutput(text) && (text.Length() == maxDigits))
                NumberWasEntered(atoi(text.GetData()));
        }
        else
            editBox->SetText(lastValidText);
    }
    return true;
}

void pawsNumberPromptWindow::NumberWasEntered(int number)
{
    if (action != NULL)
    {
        action->Execute(number);
        delete action;
        action = NULL;
    }
    parent->DeleteChild(this);
}

void pawsNumberPromptWindow::Initialize(const csString & label, int number, int minNumber, int maxNumber, iOnNumberEnteredAction * action)
{
    windowManager->SetCurrentFocusedWidget(editBox);
    
    SetLabel(label);
    this->minNumber = minNumber;
    SetMaxNumber(maxNumber);
    this->action = action;
    
    if (number != -1)
    {
        csString numberStr;
        numberStr = number;
        editBox->SetText(numberStr);
        scrollBar->SetCurrentValue(number);
    }
    else
    {
        scrollBar->SetCurrentValue(0);
        editBox->SetText("");
    }
    windowManager->SetCurrentFocusedWidget(editBox);
}


pawsNumberPromptWindow * pawsNumberPromptWindow::Create(PawsManager* windowManager, const csString & label,
                                                        int number, int minNumber, int maxNumber, iOnNumberEnteredAction * action)
{
    pawsNumberPromptWindow * w = new pawsNumberPromptWindow(windowManager);
    windowManager->GetMainWidget()->AddChild(w);
    w->PostSetup();
    w->UseBorder();
    w->SetTitle( "Amount:", "Blue Title", "center", "no" );
    windowManager->SetModalWidget(w);
    w->Initialize(label, number, minNumber, maxNumber, action);
    return w;
}

