/*
 * pawsobjectview.h - Author: Andrew Craig
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#ifndef PAWS_OBJECT_VIEW_HEADER
#define PAWS_OBJECT_VIEW_HEADER

#include <iengine/camera.h>
#include <iengine/engine.h>
#include <iengine/light.h>
#include <iengine/material.h>
#include <iengine/mesh.h>
#include <iengine/movable.h>
#include <iengine/region.h>
#include <iengine/sector.h>

#include <imesh/lighting.h>
#include <imesh/object.h>
#include <imesh/sprite3d.h>
#include <imesh/thing.h>

#include <cstool/csview.h>

#include "pawswidget.h"

/** This widget is used to view a mesh in it's own seperate world.
 */
class pawsObjectView : public pawsWidget
{
public:
    pawsObjectView( PawsManager* manager );
    ~pawsObjectView();

    /** Make a copy of this mesh to view.
     * @param wrapper  Will use the factory of this mesh to create a copy of it.
     */    
    void View( iMeshWrapper* wrapper );
    
    /** View a mesh. 
     * @param factName The name of the factory to use.
     * @param fileName The name of the file of the mesh file if the factory 
                       is not found.
     */
    void View( const char* factName, const char* fileName );
    
    /** Creates the room ( world ) for the mesh to be placed.
     */
    bool Setup(iDocumentNode* node);    
    
    /** Loads a map to use as the backdrop.
      *
      * @param map The full path name of the map to load
      * 
      * @return True if the map was loaded correctly. False otherwise.
      */
    bool LoadMap ( const char* map, const char* sector );
    
    /** Creates a default map. Creates a simple room to place object.
      * 
      * @return True if the simple map is created.
      */
    bool CreateMap();
    
    
    void Draw();

    iMeshWrapper* GetObject() { return object; }

private:
    bool CreateArea();

    csRef<iSector> stage;
    csRef<iView>   view;
    csRef<iEngine> engine;
    csRef<iMeshWrapper> object;

    csVector3 objectPos;
    static int idName;
    
    csRef<iSector> meshSector;
    csRef<iView>   meshView;  
    
    bool loadedMap;    
};
CREATE_PAWS_FACTORY( pawsObjectView );


#endif
