/*
 * pawsprefmanager.h - Author: Andrew Craig
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
// pawsprefmanager.h: interface for the pawsPrefManager class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PAWS_PREF_MANAGER_HEADER
#define PAWS_PREF_MANAGER_HEADER

#include <iutil/document.h>
#include <csutil/parray.h>

struct iObjectRegistry;
struct iVFS;
struct iGraphics2D;
struct iFont;
struct BorderDefinition;

class PawsManager; 
#define BORDER_COLOURS 5

/** Holds/Loads the prefs from a pref file. 
 */
class pawsPrefManager  
{
public:
    pawsPrefManager( PawsManager* manager );
    virtual ~pawsPrefManager();

    bool LoadPrefFile( const char* file );
    bool LoadBorderFile( const char* file );

    iFont* GetDefaultFont() { return defaultFont; }
    int    GetDefaultFontColour() { return defaultFontColour; }
    int    GetBorderColour( int index ) { return borderColours[index]; }

    // Chat window functions
    int GetChatColour() { return chatColour; }
    int GetSystemColour() { return systemColour; }
    int GetPlayerNameSaidColour() { return nameSaidColour; }
    int GetAdminColour() { return adminColour; }
    int GetTellColour() { return tellColour; }
    int GetShoutColour() { return shoutColour; }
    int GetGuildColour() { return guildColour; }
    int GetYourColour() { return yourColour; }
    int GetGroupColour() { return groupColour; }
    int GetAuctionColour() { return auctionColour; }
    
    BorderDefinition* GetBorderDefinition( const char* name );
private:

    int chatColour;
    int systemColour;
    int nameSaidColour;
    int adminColour;
    int tellColour;
    int shoutColour;
    int guildColour;
    int yourColour;
    int groupColour;
    int auctionColour;

    csRef<iObjectRegistry> objectReg;
    csRef<iVFS>             vfs;
    csRef<iDocumentSystem>  xml; 
    csRef<iGraphics2D>      graphics2D;

    csRef<iFont> defaultFont;
    int defaultFontColour;

    int borderColours[BORDER_COLOURS];
    
    void LoadChatColours( iDocumentNode* node );
    void LoadBorderColours(iDocumentNode* node);
    
    PawsManager* manager;
    
    csPDelArray<BorderDefinition> borders;
};

#endif 

