/*
 * pawsprogressbar.cpp- Author: Andrew Craig
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <cssysdef.h>
#include "pawsprogressbar.h"


pawsProgressBar::pawsProgressBar( PawsManager* manager )
                :pawsWidget( manager )
{
    totalValue = 0.0f;
    complete  = false;
    currentValue = 0.0f;
    percent      = 0.0f;
}

pawsProgressBar::~pawsProgressBar()               
{
}

bool pawsProgressBar::Setup( iDocumentNode* node )
{
    csRef<iDocumentNode> ColourNode = node->GetNode( "color" );    
    if ( ColourNode )
    {
        start_r = ColourNode->GetAttributeValueAsInt("r");
        start_g = ColourNode->GetAttributeValueAsInt("g");
        start_b = ColourNode->GetAttributeValueAsInt("b");
    }
    else
    {
	start_r = 0;
	start_g = 0;
	start_b = 25;
    }

    csRef<iDocumentNode> ColourNode2 = node->GetNode( "fadecolor" );    
    if ( ColourNode2 )
    {
        diff_r = ColourNode2->GetAttributeValueAsInt("r") - start_r;
        diff_g = ColourNode2->GetAttributeValueAsInt("g") - start_g;
        diff_b = ColourNode2->GetAttributeValueAsInt("b") - start_b;
    }
    else
    {
	diff_r = 0;
	diff_g = 0;
	diff_b = 0;
    }
    return true;
}

void pawsProgressBar::SetCurrentValue( float newValue )
{
    currentValue = newValue;
    
    percent = currentValue / totalValue;        
}

void pawsProgressBar::Draw()
{
    ClipToParent();
    int currentColor = 0;

    for(int i = 0; i <= (int)(percent*screenFrame.Width()); i++)
    {
	float pct = (float)i/screenFrame.Width();

        currentColor = graphics2D->FindRGB(start_r + (int)(pct*diff_r), 
					   start_g + (int)(pct*diff_g),
					   start_b + (int)(pct*diff_b));

        graphics2D->DrawBox ( screenFrame.xmin + i , screenFrame.ymin , 
                              1 ,screenFrame.Height(), 
                              currentColor );
    }

    pawsWidget::Draw();
}

