/*
 * pawspromptwindow.cpp - Author: Ondrej Hurt
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */


// CS INCLUDES
#include <cssysdef.h>

// COMMON INCLUDES
#include "util/log.h"

// PAWS INCLUDES
#include "pawspromptwindow.h"
#include "pawsmanager.h"
#include "pawsbutton.h"
#include "pawstextbox.h"


#define SPACING 10        //distance between inner widgets and border


pawsPromptWindow::pawsPromptWindow(PawsManager* manager) : pawsWidget(manager)
{
    label          = NULL;
    okButton       = NULL;
    cancelButton   = NULL;
    inputWidget    = NULL;
}

bool pawsPromptWindow::PostSetup()
{
    SetBackground("Standard Background");
    UseBorder("line");
    
    label = new pawsTextBox(windowManager);
    AddChild(label);
    
    okButton = new pawsButton(windowManager);
    AddChild(okButton);
    okButton->SetUpImage("Blue Title");
    okButton->SetRelativeFrameSize(60, 20);
    okButton->SetText(windowManager->Translate("OK"));
    
    cancelButton = new pawsButton(windowManager);
    AddChild(cancelButton);
    cancelButton->SetUpImage("Blue Title");
    cancelButton->SetRelativeFrameSize(60, 20);
    cancelButton->SetText(windowManager->Translate("cancel"));

    SetBackgroundAlpha(0);
    return true;
}

void pawsPromptWindow::SetLabel(const csString & label)
{
    this->label->SetText(label);
    this->label->SetSizeByText();
    LayoutWindow();
}

void pawsPromptWindow::LayoutWindow()
{
    int windowWidth, windowHeight;
    
    assert(label && okButton && cancelButton && inputWidget);
    
    windowWidth = MAX(
                        2*SPACING + label->DefaultFrame().Width(),
                        MAX(
                             2*SPACING + inputWidget->DefaultFrame().Width(),
                             3*SPACING + okButton->DefaultFrame().Width() + cancelButton->DefaultFrame().Width()
                           )
                     );
    windowHeight = 4*SPACING + label->DefaultFrame().Height() + inputWidget->DefaultFrame().Height() 
                             + okButton->DefaultFrame().Height();
    SetRelativeFrameSize(windowWidth, windowHeight);

    label->SetRelativeFramePos((windowWidth-label->DefaultFrame().Width()) / 2,
                               SPACING);

    inputWidget->SetRelativeFramePos((windowWidth-inputWidget->DefaultFrame().Width()) / 2,
                                     label->DefaultFrame().ymax + SPACING);

    okButton->SetRelativeFramePos((windowWidth-okButton->DefaultFrame().Width()-cancelButton->DefaultFrame().Width()-SPACING) / 2,
                                  inputWidget->DefaultFrame().ymax + SPACING);

    cancelButton->SetRelativeFramePos(okButton->DefaultFrame().xmax + SPACING,
                                      okButton->DefaultFrame().ymin);

    SetAppropriatePos();
}

void pawsPromptWindow::SetAppropriatePos()
{
    csPoint mouse;
    int x, y;
    int width, height;
    
    width   =  screenFrame.Width();
    height  =  screenFrame.Height();
    
    mouse = windowManager->GetMouse()->GetPosition();
    x = mouse.x - width  / 2;
    y = mouse.y - height / 2;
    MoveTo(x, y);
    MakeFullyVisible();
}
