/*
 * pawsstringpromptwindow.h - Author: Ondrej Hurt
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef PAWS_STRING_PROMPT_WINDOW_HEADER
#define PAWS_STRING_PROMPT_WINDOW_HEADER

#include <csutil/list.h>
#include <iutil/document.h>
#include "pawspromptwindow.h"

class pawsButton;

class iOnStringEnteredAction
{
public:
    virtual void Execute(const csString & string) = 0;
        // can be -1
};

/** 
 * pawsStringPromptWindow is window that lets the user enter string.
 */
class pawsStringPromptWindow : public pawsPromptWindow
{
public:
    pawsStringPromptWindow(PawsManager* manager);
    
	//from pawsWidget:
    virtual bool PostSetup();
    bool OnButtonPressed( int mouseButton, int keyModifier, pawsWidget* widget );
    virtual bool OnKeyDown( int keyCode, int keyChar, int modifiers );
    
    void Initialize(const csString & label, const csString & string, bool multiline, 
                    int width, int height, iOnStringEnteredAction * action);
    
    static pawsStringPromptWindow * Create(PawsManager* windowManager, 
                                           const csString & label, const csString & string, 
                                           bool multiline, int width, int height, iOnStringEnteredAction * action);

protected:
    /** Executes action with 'text' as parameter and destroys window */
    void CloseWindow(const csString & text);
    
    /** Executes action with text entered by user as parameter and destroys window */
    void CloseWindow();
    
    iOnStringEnteredAction * action;
};

CREATE_PAWS_FACTORY(pawsStringPromptWindow)


#endif
