/*
* binarytree.cpp by Keith Fulton <keith@paqrat.com>
*
* Copyright (C) 2001 PlaneShift Team (info@planeshift.it, 
* http://www.planeshift.it)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation (version 2 of the License)
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*/
#ifndef __BINARYTREE_CPP__
#define __BINARYTREE_CPP__

#ifndef __BINARYTREE_H__

#include <cssysdef.h>
#include "binarytree.h"
#endif


template <class T>
BinaryTree<T>::BinaryTree()
{
    pRoot = NULL;
    lCount = 0;
}


template <class T>
BinaryTree<T>::~BinaryTree()
{
    delete pRoot;
}

template <class T>
void BinaryTree<T>::Clear()
{
    delete pRoot;
    pRoot = NULL;
    lCount = 0;
}


template <class T>
BinaryTreeNode<T> *BinaryTree<T>::Add(T *pT,int iOwner)
{
    int x=1;

    if (!pRoot)
        pRoot = new BinaryTreeNode<T>(pT,iOwner);
    else
	{
        x = pRoot->AddOffspring(pT,iOwner);
		while (pRoot->parent)
			pRoot = pRoot->parent;
	}
	pRoot->color = true;

    if (x)
        lCount++;

    return NULL;
}

template <class T>
int BinaryTree<T>::Delete(T *pT)
{
    if (!pRoot)
        return 0;

	BinaryTreeNode<T> *newRoot;

	if(!(*pRoot->pObj == *pT))
		newRoot = pRoot;
	else
	{
		if (pRoot->pLeft)
			newRoot = pRoot->pLeft;
		else
			newRoot = pRoot->pRight;
	}
	int x = pRoot->Delete(pT);
	if (x == -1)
	{
		lCount--;

		// Needed in case root was deleted
		if (newRoot)
		{
			while(newRoot->parent)
				newRoot = newRoot->parent;
		}
		pRoot = newRoot;
	}

	return x;
}


template <class T>
T *BinaryTree<T>::Find(T *pT) const
{
    const BinaryTreeNode<T> *pNode = GetNodeRef(pT);

    if (pNode)
        return pNode->pObj;
    else
        return NULL;
}

template <class T>
T *BinaryTree<T>::FindPartial(T *pPartialT,int bGuaranteeUnique) const
{
    if (!pRoot)
        return NULL;
    else
    {
        const BinaryTreeNode<T> *pNode = pRoot->FindPartial(pPartialT,bGuaranteeUnique);

        if (pNode)
            return pNode->pObj;
        else
            return NULL;
    }
}



template <class T>
const BinaryTreeNode<T> *BinaryTree<T>::GetNodeRef(T *pT) const
{
    if (pRoot)
        return pRoot->Find(pT);
    else
        return NULL;
}

template <class T>
BinaryTreeNode<T>::BinaryTreeNode(T *pT,bool bOwnObject)
{
    pObj   = pT;
    iRefCount = 1;
    bOwner = bOwnObject;
    pLeft  = NULL;
    pRight = NULL;
	parent = NULL;
	color = false;
}

template <class T>
BinaryTreeNode<T>::~BinaryTreeNode()
{
    if (bOwner)
        delete pObj;
    if (pLeft)
        delete pLeft;
    if (pRight)
        delete pRight;
}

template <class T>
int BinaryTreeNode<T>::Delete(T *pKey)
{
    BinaryTreeNode<T> *pNode;
	BinaryTreeNode<T> *pNodeX; 
	BinaryTreeNode<T> *pNodeY; 

	pNode = const_cast<BinaryTreeNode<T> *> (Find(pKey));

    if (!pNode)
        return 0; // Node not found.

    pNode->iRefCount--;
    if (pNode->iRefCount)
        return pNode->iRefCount; // Node deleted but still present.

	// Find node to move or remove
	if (!pNode->pLeft || !pNode->pRight)
		pNodeY = pNode;
	else
	{
		if (pNode->pRight)
		{
			pNodeY = pNode->pRight;
			while (pNodeY->pLeft)
				pNodeY = pNodeY->pLeft;
		}
		else
		{
			BinaryTreeNode<T> *pTemp = pNode->parent;
			BinaryTreeNode<T> *pTemp2 = pNode;
			while (pTemp && pTemp2 == pTemp->pRight)
			{
				pTemp2 = pTemp;
				pTemp = pTemp->parent;
			}
			pNodeY = pTemp;
		}
	}

	if (pNodeY->pLeft)
		pNodeX = pNodeY->pLeft;
	else
		pNodeX = pNodeY->pRight;

	if (pNodeX)
		pNodeX->parent = pNodeY->parent;
	
	if (pNodeY->parent)
	{
        if (pNodeY == pNodeY->parent->pLeft)
			pNodeY->parent->pLeft = pNodeX;
		else
			pNodeY->parent->pRight = pNodeX;
	}

	if (pNodeY != pNode)
	{
		pNode->pObj = pNodeY->pObj;
		pNode->iRefCount = pNodeY->iRefCount;
		pNode->bOwner = pNodeY->bOwner;
	}
	if (pNodeY->color)
		DeleteFix(pNodeX, pNodeY->parent);

	pNodeY->pLeft = NULL;
	pNodeY->pRight = NULL;
	delete pNodeY;

    return -1;
}

template <class T>
const BinaryTreeNode<T> *BinaryTreeNode<T>::Find(T *pT) const
{
    if (*this->pObj == *pT)
        return this;

    if (*pT < *this->pObj)
    {
        if (pLeft)
            return pLeft->Find(pT);

        return NULL;
    }
    else
    {
        if (pRight)
            return pRight->Find(pT);

        return NULL;
    }
}

template <class T>
const BinaryTreeNode<T> *BinaryTreeNode<T>::FindPartial(T *pT,int bGuaranteeUnique) const
{
    int iEqu=0;

    if (! (iEqu = this->pObj->PartialEquals(*pT)) )
    {
        if (!bGuaranteeUnique)
            return this;

        if (pLeft)
        {
            if (pLeft->FindPartial(pT,0))
                return NULL;
        }
        if (pRight)
        {
            if (pRight->FindPartial(pT,0))
                return NULL;
        }
        return this;
    }
    else
    {
        if (iEqu > 0)
        {
            if (pLeft)
                return pLeft->FindPartial(pT,bGuaranteeUnique);
            else
                return NULL;
        }
        else
        {
            if (pRight)
                return pRight->FindPartial(pT,bGuaranteeUnique);
            else
                return NULL;
        }
    }
}

//template <class T>
//void BinaryTreeNode<T>::AddSubTree(BinaryTreeNode<T> *pTree)
//{
//    if (!pTree || *pTree->pObj == *pObj)
//        return;
//
//    if (*pTree->pObj < *pObj)
//    {
//        if (pLeft)
//            pLeft->AddSubTree(pTree);
//        else
//		{
//            pLeft = pTree;
//			pTree->parent = this;
//			pTree->color = false;
//			Rebalance(pTree);
//		}
//
//        return;
//    }
//    else
//    {
//        if (pRight)
//            pRight->AddSubTree(pTree);
//        else
//		{
//            pRight = pTree;
//			pTree->parent = this;
//			pTree->color = false;
//			Rebalance(pTree);
//		}
//
//        return;
//    }
//}

template <class T>
int BinaryTreeNode<T>::AddOffspring(T *pNew,int iOwner)
{
    if (*pNew == *pObj)
    {
        iRefCount++;
        return 0;   // Didn't really add anything
    }

    if (*pNew < *pObj)
    {
        if (pLeft)
            return pLeft->AddOffspring(pNew,iOwner);
        else
        {
            pLeft = new BinaryTreeNode<T>(pNew,iOwner);
			pLeft->parent = this;
			Rebalance(pLeft);
            return 1;
        }
    }
    else
    {
        if (pRight)
            return pRight->AddOffspring(pNew,iOwner);
        else
        {
            pRight = new BinaryTreeNode<T>(pNew,iOwner);
			pRight->parent = this;
			Rebalance(pRight);
            return 1;
        }
    }
}

template <class T>
void BinaryTreeNode<T>::Rebalance(BinaryTreeNode<T> *pTree)
{
	while(pTree->parent && !pTree->parent->color)
	{
		if (pTree->parent == pTree->parent->parent->pLeft) // Left child
		{
			BinaryTreeNode<T> *pNode = pTree->parent->parent->pRight;
			if (!pNode || pNode->color)
			{
				if (pTree == pTree->parent->pRight)
				{
					pTree = pTree->parent;
					LRotate(pTree);
				}
				pTree->parent->color = true;
				pTree->parent->parent->color = false;
				RRotate(pTree->parent->parent);
			}
			else
			{
				pTree->parent->color = true;
				pNode->color = true;
				pTree->parent->parent->color = false;
				pTree = pTree->parent->parent;
			}
		}
		else
		{
			BinaryTreeNode<T> *pNode = pTree->parent->parent->pLeft;
			if (!pNode || pNode->color)
			{
				if (pTree == pTree->parent->pLeft)
				{
					pTree = pTree->parent;
					RRotate(pTree);
				}
				pTree->parent->color = true;
				pTree->parent->parent->color = false;
				LRotate(pTree->parent->parent);
			}
			else
			{
				pTree->parent->color = true;
				pNode->color = true;
				pTree->parent->parent->color = false;
				pTree = pTree->parent->parent;
			}
		}

	}
}

template <class T>
void BinaryTreeNode<T>::LRotate(BinaryTreeNode<T> *pTree)
{
	BinaryTreeNode<T> *pNode = pTree->pRight;
	pTree->pRight = pNode->pLeft;
	if (pNode->pLeft)
		pNode->pLeft->parent = pTree;
	pNode->parent = pTree->parent;
	if (pTree->parent)
	{
		if (pTree == pTree->parent->pLeft)
			pTree->parent->pLeft = pNode;
		else
			pTree->parent->pRight = pNode;
	}
	pNode->pLeft = pTree;
	pTree->parent = pNode;
}




template <class T>
void BinaryTreeNode<T>::RRotate(BinaryTreeNode<T> *pTree)
{
	BinaryTreeNode<T> *pNode = pTree->pLeft;
	pTree->pLeft = pNode->pRight;
	if (pNode->pRight)
		pNode->pRight->parent = pTree;
	pNode->parent = pTree->parent;
	if (pTree->parent)
	{
		if (pTree == pTree->parent->pLeft)
			pTree->parent->pLeft = pNode;
		else
			pTree->parent->pRight = pNode;
	}
	pNode->pRight = pTree;
	pTree->parent = pNode;
}

template <class T>
void BinaryTreeNode<T>::DeleteFix(BinaryTreeNode<T> *pNodeX, BinaryTreeNode<T> *pParent)
{
	// pParent is needed in case pNodeX (deleted node) is NULL
	while (pParent && (!pNodeX || pNodeX->color))
	{
		if (pNodeX == pParent->pLeft)
		{
			BinaryTreeNode<T> *pNode = pParent->pRight;
			if (!pNode->color)
			{
				pNode->color = true;
				pParent->color = false;
				LRotate(pParent);
				pNode = pParent->pRight;
			}
			if((!pNode->pLeft || pNode->pLeft->color) && (!pNode->pRight || pNode->pRight->color))
			{
				pNode->color = false;
				pNodeX = pParent;
				pParent = pNodeX->parent;
			}
			else
			{
				if (!pNode->pRight || pNode->pRight->color)
				{
					if (pNode->pLeft)
						pNode->pLeft->color = true;
					pNode->color = false;
					RRotate(pNode);
					pNode = pParent->pRight;
				}
				pNode->color = pParent->color;
				pParent->color = true;
				if (pNode->pRight)
					pNode->pRight->color = true;
				LRotate(pParent);
				// Set pNodeX to the root
				while (pParent)
				{
					pNodeX = pParent;
					pParent = pNodeX->parent;
				}
			}
		}
		else
		{
			BinaryTreeNode<T> *pNode = pParent->pLeft;
			if (!pNode->color)
			{
				pNode->color = true;
				pParent->color = false;
				RRotate(pParent);
				pNode = pParent->pLeft;
			}
			if((!pNode->pRight || pNode->pRight->color) && (!pNode->pLeft || pNode->pLeft->color))
			{
				pNode->color = false;
				pNodeX = pParent;
				pParent = pNodeX->parent;
			}
			else
			{
				if (!pNode->pLeft || pNode->pLeft->color)
				{
					if (pNode->pRight)
						pNode->pRight->color = true;
					pNode->color = false;
					LRotate(pNode);
					pNode = pParent->pLeft;
				}
				pNode->color = pParent->color;
				pParent->color = true;
				if (pNode->pLeft)
					pNode->pLeft->color = true;
				RRotate(pParent);
				// Set pNodeX to the root
				while (pParent)
				{
					pNodeX = pParent;
					pParent = pNodeX->parent;
				}
			}
		}
	}
	if (pNodeX)
		pNodeX->color = true;
}

template <class T>
BinaryTreeIterator<T>::BinaryTreeIterator(const BinaryTree<T> *pTree,
                                          iterorder iOrder)
{
    pMyTree    = pTree;
    pNodeList  = new BinaryTreeNode<T> *[2000];
    iDepth     = -1;
    lIndex     = -1;
    iIterOrder = iOrder;
}

template <class T>
BinaryTreeIterator<T>::~BinaryTreeIterator()
{
    delete[] pNodeList;
}

template <class T>
T *BinaryTreeIterator<T>::First(void)
{
    if (!pMyTree->pRoot)
    {
        iDepth = -1;
        lIndex = -1;
        return NULL;
    }

    switch(iIterOrder)
    {
    case ITER_PREORDER:
        {
            iDepth=0;
            pNodeList[iDepth]=pMyTree->pRoot;
            lIndex = 0;
            return pNodeList[iDepth]->pObj;
        }
    case ITER_INORDER:
        {
            iDepth = 0;
            pNodeList[iDepth] = pMyTree->pRoot;

            while (pNodeList[iDepth]->pLeft)
            {
                pNodeList[iDepth+1] = pNodeList[iDepth]->pLeft;
                iDepth++;
            }
            lIndex = 0;
            return pNodeList[iDepth]->pObj;
        }
    }

    return NULL;
}

template <class T>
T *BinaryTreeIterator<T>::Last(void)
{
    if (!pMyTree->pRoot)
    {
        iDepth = -1;
        lIndex = -1;
        return NULL;
    }

    // Last node same for PRE,IN order
    iDepth = 0;
    pNodeList[0] = pMyTree->pRoot;

    while (pNodeList[iDepth]->pRight)
        pNodeList[++iDepth] = pNodeList[iDepth-1]->pRight;

    lIndex = pMyTree->lCount;
    //    bWentLeft = 1;
    return pNodeList[iDepth]->pObj;
}

template <class T>
T *BinaryTreeIterator<T>::Next(void)
{
    if (pMyTree->pRoot == NULL || lIndex==-1)
        return NULL;
    
    switch(iIterOrder)
    {
    case ITER_PREORDER:
        {
            if (pNodeList[iDepth]->pLeft)
            {
                if (pNodeList[iDepth]->pRight)
                {
                    pNodeList[iDepth+1] = pNodeList[iDepth]->pLeft;
                    iDepth++;
                }
                else
                    pNodeList[iDepth] = pNodeList[iDepth]->pLeft;

                lIndex++;
                return pNodeList[iDepth]->pObj;
            }
            else if (pNodeList[iDepth]->pRight)
            {
                pNodeList[iDepth] = pNodeList[iDepth]->pRight;
                lIndex++;
                return pNodeList[iDepth]->pObj;
            }
            else // no children
            {
                iDepth--;
                if (iDepth > -1)
                {
                    if (pNodeList[iDepth]->pRight)
                    {
                        pNodeList[iDepth] = pNodeList[iDepth]->pRight;
                        lIndex++;
                        return pNodeList[iDepth]->pObj;
                    }
                    else
                    {
                        lIndex = -1;
                        iDepth = -1;
                        return NULL;
                    }
                }
                else
                {
                    lIndex=-1;
                    return NULL;
                }
            }
        }
    case ITER_INORDER:
        {
            if (pNodeList[iDepth]->pRight)
            {
                pNodeList[iDepth+1] = pNodeList[iDepth]->pRight;
                iDepth++;
                while (pNodeList[iDepth]->pLeft)
                {
                    pNodeList[iDepth+1] = pNodeList[iDepth]->pLeft;
                    iDepth++;
                }
                lIndex++;
                return pNodeList[iDepth]->pObj;
            }
            else
            {
                iDepth--;
                while (iDepth>=0 && pNodeList[iDepth]->pRight == pNodeList[iDepth+1])
                    iDepth--;
                lIndex++;
                if (iDepth>=0)
                    return pNodeList[iDepth]->pObj;
                else
                    return NULL;
            }
        }
    }
    return NULL;
}

template <class T>
T *BinaryTreeIterator<T>::Prev(void)
{
    printf ("***BinaryTreeIterator::Prev not implemented yet!!!!\n");
    return NULL; // not implemented yet
}

template <class T>
T *BinaryTreeIterator<T>::Curr(void)
{
    if (iDepth<0)
        return NULL;

    return pNodeList[iDepth]->pObj;
}

template <class T>
T *BinaryTreeIterator<T>::Find(T *key)
{
    if (pMyTree->pRoot == NULL)
        return NULL;

    iDepth = 0;
    pNodeList[iDepth] = pMyTree->pRoot;

    while (!(*(pNodeList[iDepth]->pObj) == *key))
    {
        if (*(pNodeList[iDepth]->pObj) < *key)
            pNodeList[iDepth+1] = pNodeList[iDepth]->pRight;
        else
            pNodeList[iDepth+1] = pNodeList[iDepth]->pLeft;

        iDepth++;

        if (pNodeList[iDepth]==NULL) // not found and fell off the end of the tree
        {
            iDepth = -1;
            lIndex = -1;
            return NULL;
        }
    }
    lIndex = 0;
    return pNodeList[iDepth]->pObj;
}

#endif

