/*
* psplog.cpp -- Christophe Painchaud aka Atanor, DaSH <dash@ionblast.net>
*
* Copyright (C) 2001 PlaneShift Team (info@planeshift.it,
* http://www.planeshift.it)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation (version 2 of the License)
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*/
#include <cssysdef.h>

#include <iutil/objreg.h>
#include <ivaria/reporter.h>
#include "log.h"

namespace pslog
{

iObjectRegistry* logger;
bool disp_flag[MAX_FLAGS];

char *flagnames[] = {   "LOG_ANY",
			"LOG_WEATHER" ,
			"LOG_SPAWN"   ,
			"LOG_CELPERSIST",
			"LOG_PAWS"      ,
			"LOG_GROUP"     ,
			"LOG_CHEAT"     ,
			"LOG_LINMOVE"   ,
			"LOG_SPELLS",
			"LOG_NEWCHAR",
			"LOG_SUPERCLIENT",
			"LOG_EXCHANGES",
			"LOG_ADMIN",
			"LOG_STARTUP",
			"LOG_CHARACTER",
			"LOG_CONNECTIONS",
			"LOG_CHAT",
            "LOG_NET",
            "LOG_LOAD",
            "LOG_NPC",
            "LOG_TRADE"
		    };


void LogMessage (const char* file, int line, const char* function,
	         int severity, int type, const char* msg, ...)
{
    if (logger == 0)
	return;
    if (!disp_flag[type])
	return;

    va_list arg;
    va_start(arg, msg);

    char msgid[5000];
    snprintf (msgid, 5000, "<%s:%d> %s",
      file, line, function);

    csReportV (logger, severity, msgid, msg, arg);

    va_end(arg);
}


void Initialize(iObjectRegistry* object_reg)
{
    logger = object_reg;

    static bool initialized = false;
    if (initialized) return;
    initialized = true;

    for (int i = 0; i < MAX_FLAGS; i++)
    {
        disp_flag[i] = false;
    }

    disp_flag[LOG_ANY]         = true;
    disp_flag[LOG_CONNECTIONS] = true;
    disp_flag[LOG_CHAT]        = true;
    disp_flag[LOG_NET]         = true;
    disp_flag[LOG_CHARACTER]   = true;
    disp_flag[LOG_NEWCHAR]     = true;
}

void SetFlag(const char *name,bool flag)
{
    for (int i=0; i<MAX_FLAGS; i++)
    {
	if (!strcmp(flagnames[i],name))
	{
	    disp_flag[i] = flag;
	    printf("%s Flag updated.\n",name);
	    break;
	}
    }
}

void DisplayFlags(const char *name)
{
    for (int i=0; i<MAX_FLAGS; i++)
    {
	if (!name || !strcmp(flagnames[i],name))
	{
	    printf("%s = %s.\n",flagnames[i],disp_flag[i]?"true":"false");
	}
    }
}






} // end of namespace pslog

