/*
* pslog.h -- Christophe Painchaud aka Atanor, DaSH <dash@ionblast.net>
*
* Copyright (C) 2001 PlaneShift Team (info@planeshift.it,
* http://www.planeshift.it)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation (version 2 of the License)
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*/

#ifndef __PSUTIL_LOG_H__
#define __PSUTIL_LOG_H__

#include "ivaria/reporter.h"

struct iObjectRegistry;

enum
{
    LOG_ANY     ,
    LOG_WEATHER ,
    LOG_SPAWN   ,
    LOG_CELPERSIST,
    LOG_PAWS      ,
    LOG_GROUP     ,
    LOG_CHEAT     ,
    LOG_LINMOVE   ,
    LOG_SPELLS,
    LOG_NEWCHAR,
    LOG_SUPERCLIENT,
    LOG_EXCHANGES,
    LOG_ADMIN,
    LOG_STARTUP,
    LOG_CHARACTER,
    LOG_CONNECTIONS,
    LOG_CHAT,
    LOG_NET,
    LOG_LOAD,
    LOG_NPC,
    LOG_TRADE,
    MAX_FLAGS
// NOTE: Remember to update the flagnams table in log.cpp when addning new entries
};

namespace pslog
{

extern iObjectRegistry* logger;
extern bool disp_flag[MAX_FLAGS];

void LogMessage (const char* file, int line, const char* function,
	         int severity, int type, const char* msg, ...);
void Initialize(iObjectRegistry* object_reg);
void SetFlag(const char *name,bool flag);
void DisplayFlags(const char *name=NULL);

// Debug
#define Debug1(type,a) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_DEBUG, type, a)
#define Debug2(type,a,b) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_DEBUG, type, a, b)
#define Debug3(type, a,b,c) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_DEBUG, type, a, b, c)
#define Debug4(type,a,b,c,d) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_DEBUG, type, a, b, c, d)
#define Debug5(type, a,b,c,d,e) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_DEBUG, type, a, b, c, d, e)
#define Debug6(type, a,b,c,d,e,f) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_DEBUG, type, a, b, c, d, e, f)
#define Debug7(type, a,b,c,d,e,f,g) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_DEBUG, type, a, b, c, d, e, f, g)

#define Notify1(type, a) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_NOTIFY, type, a)
#define Notify2(type, a,b) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_NOTIFY, type, a, b)
#define Notify3(type, a,b,c) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_NOTIFY, type, a, b, c)
#define Notify4(type, a,b,c,d) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_NOTIFY, type, a, b, c, d)
#define Notify5(type, a,b,c,d,e) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_NOTIFY, type, a, b, c, d, e)
#define Notify6(type, a,b,c,d,e,f) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_NOTIFY, type, a, b, c, d, e, f)


#define Warning1(type, a) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_WARNING, type, a)
#define Warning2(type, a,b) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_WARNING, type, a, b)
#define Warning3(type, a,b,c) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_WARNING, type, a, b, c)
#define Warning4(type, a,b,c,d) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_WARNING, type, a, b, c, d)
#define Warning5(type, a,b,c,d,e) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_WARNING, type, a, b, c, d, e)
#define Warning6(type, a,b,c,d,e,f) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_WARNING, type, a, b, c, d, e, f)


#define Error1(a) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_ERROR, LOG_ANY,a)
#define Error2(a,b) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_ERROR, LOG_ANY,a, b)
#define Error3(a,b,c) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_ERROR, LOG_ANY,a, b, c)
#define Error4(a,b,c,d) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_ERROR, LOG_ANY,a, b, c, d)
#define Error5(a,b,c,d,e) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_ERROR, LOG_ANY,a, b, c, d, e)
#define Error6(a,b,c,d,e,f) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_ERROR, LOG_ANY,a, b, c, d, e, f)

#define Bug1(a) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_BUG, LOG_ANY,a)
#define Bug2(a,b) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__ ,\
    CS_REPORTER_SEVERITY_BUG, LOG_ANY,a, b)
#define Bug3(a,b,c) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_BUG, LOG_ANY,a, b, c)
#define Bug4(a,b,c,d) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_BUG, LOG_ANY,a, b, c, d)
#define Bug5(a,b,c,d,e) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__ \
    CS_REPORTER_SEVERITY_BUG, LOG_ANY,a, b, c, d, e),
#define Bug6(a,b,c,d,e,f) \
    pslog::LogMessage (__FILE__, __LINE__, __FUNCTION__, \
    CS_REPORTER_SEVERITY_BUG, LOG_ANY,a, b, c, d, e, f)

} // end of namespace pslog

#endif

