#include <cssysdef.h>

#include <string.h>
#include <ctype.h>

#include "psstring.h"
#include "strutil.h"

psString::psString() : csString()
{
}

psString::psString(const char* pStr) : csString(pStr)
{
}

psString::psString(const csString& str) : csString(str)
{
}

int psString::FindSubString(const char *sub, size_t start, bool caseInsense)
{
    size_t lensub = strlen(sub);
    if ( IsEmpty() || !lensub || lensub>Length() )
	return -1;

    if ( caseInsense )
    {
		while ( start <= Length() - lensub )
		{
			if (strncasecmp(sub, Data + start, lensub) != 0)
				start++;
			else
				return start;
		}
		return -1;
	}
	else
	{
		char* pWhere = strstr(Data + start, sub);
		if (pWhere)
			return pWhere - Data;
		else
			return -1;
	}
}

int psString::FindSubStringReverse(psString& sub, size_t start, bool caseInsense)
{
    if ( IsEmpty() || sub.IsEmpty() || sub.Length()>Length() )
	return -1;

    if ( caseInsense )
    {
        while ( start >= 0 + sub.Length() )
        {
            char* pWhere = Data + start - sub.Length();
            size_t x;
            for ( x = 0; x < sub.Length(); x++ )
            {
                if ( toupper(pWhere[x]) != toupper(sub[x]) )
                break;
            }
            if ( x < sub.Length() )
                start--;
            else
                return pWhere-Data;
        }
        return -1;
    }
    else
    {
        while ( start >= 0 + sub.Length() )
        {
            char* pWhere = Data + start - sub.Length();
            size_t x;
            x=0;
            for ( x = 0; x < sub.Length(); x++ )
            {
                if ( pWhere[x] != sub[x] )
                break;
            }
            if ( x < sub.Length() )
                start--;
            else
                return pWhere-Data;
        }
        return -1;
    }
}

void psString::GetSubString(psString& str, size_t from, size_t to)
{
	str.Clear();

    if ( from > Size || from > to )
    	return;

    size_t len = to-from;
    str.Append ( ((const char*) *this) + from, len);
}

void psString::GetWord(size_t pos, psString &buff, bool wantPunct)
{
    size_t start = pos;
    size_t end   = pos;

    if (pos > Size)
    {
	buff="";
	return;
    }

    // go back to the beginning of the word
    while (start > 0 && (!isspace(Data[start])) &&
           (wantPunct || !ispunct(Data[start])) 
          )
	start--;
    if (isspace(Data[start]) || (!wantPunct && ispunct(Data[start])))
        start++;

    // search end of the word
    while (end<Size && (!isspace(Data[end])) &&
           (wantPunct || !ispunct(Data[end])) 
          )
	end++;

    GetSubString(buff,start,end);
}

void psString::GetWordNumber(int which,psString& buff)
{
    buff = ::GetWordNumber((csString) *this, which);
}


void psString::RemoveSubString(size_t from, size_t to)
{
    if (from >= Size || from>to)
	return;
    DeleteAt(from, to-from);
}


//---------------------------------------------------------------------------
bool psString::ReplaceSubString(psString& what, psString& with)
{
    int where;

    if ( (where = FindSubString(what)) > -1 )
    {
	size_t pos = where;
	DeleteAt(where, what.Length());
	Insert(pos, with);
	return true;
    }
    else
	return false;
}

int psString::FindCommonLength(psString& other)
{
    if (!Data || !other.Data)
	return 0;

    size_t i;
    for (i=0; i<Length(); i++)
    {
	    if (Data[i] != other.Data[i])
	        return i;
    }
    return i;
}

psString& psString::AppendFormat(char * format, ... )
{
    va_list args;

    va_start(args, format);
    psString tmp;
    Append(tmp.FormatV(format,args));
    va_end(args);

    return *this;
    
}



