/*
 * psstring.h
 *
 * Copyright (C) 2001 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * A slightly improved string over csString.  Adds a couple of functions
 * that will be needed by the XML parser.
 */
#ifndef PS_STRING_H
#define PS_STRING_H

#include <csutil/csstring.h>

#define XML_CASE_INSENSITIVE true
#define XML_CASE_SENSITIVE   false


class psString : public csString
{
public:
    psString();
    psString(const char* str);
    psString(const csString& str);

    int FindSubString(const char *sub, size_t start=0, bool caseInsense=XML_CASE_SENSITIVE);
    int FindSubStringReverse(psString& sub, size_t start, bool caseInsense=XML_CASE_SENSITIVE);
    void GetSubString(psString& str, size_t from, size_t to);

    enum
    {
    	NO_PUNCT=0,
	    INCLUDE_PUNCT=1
    };
    void GetWord(size_t pos,psString &buff,bool wantPunct=INCLUDE_PUNCT);
    void GetWordNumber(int which,psString& buff);

    void RemoveSubString(size_t from, size_t to);
    bool ReplaceSubString(psString& what, psString& with);

    bool operator==(psString& other)
    { return strcmp(Data,other.Data)==0; }

    bool operator<(psString& other)
    { return strcmp(Data,other.Data)<0;  }

    int PartialEquals(psString& other)
    { return strncmp(Data,other.Data,other.Length()); }

    int FindCommonLength(psString& other);

    psString& AppendFormat(char * format, ... );
};

#endif

