/*
 * strutil.cpp by Matze Braun <MatzeBraun@gmx.de>
 *
 * Copyright (C) 2001 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#include <cssysdef.h>
#include <ctype.h>

#include <csutil/csstring.h>
#include <csutil/array.h>

#include "strutil.h"

csString GetWordNumber(const csString& str, int number, size_t startpos)
{
    CS_ASSERT(number>0);

    size_t pos=startpos;

    if (pos>=str.Length())
        return "";

    //skip leading spaces
    if (isspace(str[pos]))
	number++;

    while (number>1 && pos < str.Length())
    {
	if (isspace(str[pos++]))
	{
	    // Skip continuous spaces
	    while (pos<str.Length() && isspace(str[pos]))
		pos++;
	    number--;
	}
    }
    size_t pos2=pos+1;
    while (pos2 < str.Length() && !isspace(str[pos2]))
	pos2++;
    
    csString erg;
    erg.Replace(((const char*)str)+pos,pos2-pos);

    return erg;
}


CmdWords::CmdWords(const csString & cmd)
{
    int pos;

    pos = 0;
    while (pos < cmd.Length())
    {
        SkipSpaces(cmd, pos);
        if (pos < cmd.Length())
            pos = AddWord(cmd, pos);
    }
}

void CmdWords::SkipSpaces(const csString & cmd, int & pos)
{
    while (pos<cmd.Length()  &&  isspace(cmd.GetAt(pos)))
        pos++;
}

int CmdWords::AddWord(const csString & cmd, int pos)
{
    int end=pos;
    while (end<cmd.Length()  &&  !isspace(cmd.GetAt(end)))
        end++;
    words.Push(cmd.Slice(pos, end-pos));
    return end;
}

int CmdWords::GetCount()
{
    return words.Length();
}

csString CmdWords::Get(int wordNum)
{
    if (wordNum<0  ||  wordNum>=words.Length())
        return "";
    else
        return words[wordNum];
}

csString CmdWords::operator[](int wordNum)
{
    return Get(wordNum);
}

int CmdWords::GetInt(int wordNum)
{
    return atoi(Get(wordNum).GetData());
}

csString CmdWords::GetTail(int wordNum)
{
    csString tail;

    for (int currWordNum = wordNum; currWordNum < words.Length(); currWordNum++)
        tail += words[currWordNum];
    return tail;
}



bool psContain(const csString& str, const csArray<csString>& strs)
{
    for (int i = 0; i < strs.Length(); i++){
        if (strstr(
                      str? str:"",
                      strs[i]? strs[i]:""
                  )
           )
          return true;
    }
    return false;
}

bool psSentenceContain(const csString& sentence,const csString& word)
{
    CmdWords words(sentence);
    int n = 1;
    csString currentWord;

    do {
        currentWord = words[n-1];
        if (currentWord == "")
        {
            return false;
        }
        n++;
    } while (currentWord != word);
    return true;
}


char* PS_GetFileName(char* path)
{
    int pos = strlen(path);
    for ( ; pos>=0; pos--)
    {
    	if (path[pos] == '/')
	    break;
    }
    
    return path+pos+1;
}

csArray<csString> psSplitt(csString& str, char delimer)
{
    csArray<csString> splitt;

    char *curr = str.GetData();
    char *next = curr;
    while (next = strchr(curr,delimer))
    {
        *next = '\0';
        splitt.Push(csString(curr));
        *next = delimer;
        curr = ++next;
    }
    splitt.Push(csString(curr));
    return splitt;
}
