/*
 * widgetconfigwindow.cpp - Author: Ian Donderwinkel
 *
 * Copyright (C) 2003 PlaneShift Team (info@planeshift.it,
 * http://www.planeshift.it)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation (version 2 of the License)
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */


// CS INCLUDES
#include <cssysdef.h>

#include "pawswidget.h"
#include "pawscrollbar.h"
#include "pawstextbox.h"
#include "pawsprogressbar.h"
#include "pawsbutton.h"
#include "pawsmainwidget.h"
#include "pawswidget.h"
#include "pawsmanager.h"
#include "util/psstring.h"

#include "widgetconfigwindow.h"

//-----------------------------------------------------------------------------
//                            class WidgetConfigWindow
//-----------------------------------------------------------------------------
WidgetConfigWindow::WidgetConfigWindow( PawsManager* manager) : pawsWidget(manager)
{
    SetAlwaysOnTop(true);
    windowManager->SetCurrentFocusedWidget(this);  
	configWidget = NULL;
};


bool WidgetConfigWindow::OnScroll( int direction, pawsScrollBar* widget )
{
	if (!configWidget)
		return true;

    psString pct;
    if (widget == scrollBarMinAlpha)
    {
        // get current minimum alpha value from scrollbar
        float value = scrollBarMinAlpha->GetCurrentValue();

        // this minimum value must be less than the maximum alpha value 
        if (value<scrollBarMaxAlpha->GetCurrentValue())
        {
            // update the progress bar
            progressBarMinAlpha->SetCurrentValue(value);
            // set the alpha value of the widget to this value
            // to give the user an idea of how this will look
            // DO NOT call draw() here, it is not necessary
            configWidget->SetBackgroundAlpha((int)value);

            // show alpha in %
            pct.Format("%.0f%%",value/2.55f);
            textMinAlphaPct->SetText(pct);
	
            currentMinAlpha = value;
        } else	
        {
            // undo scrollbar action
            scrollBarMinAlpha->SetCurrentValue(currentMinAlpha);
        }
    } else if (widget == scrollBarMaxAlpha)
    {
        // update the scroll and progress bar for max alpha
        float value = scrollBarMaxAlpha->GetCurrentValue();

        // max alpha must not be less than min alpha
        if (value>scrollBarMinAlpha->GetCurrentValue())
        {
            progressBarMaxAlpha->SetCurrentValue(value);

            configWidget->SetBackgroundAlpha((int)value);

            pct.Format("%.0f%%",value/2.55f);
            textMaxAlphaPct->SetText(pct);

            currentMaxAlpha = value;
        } else	
        {
            scrollBarMaxAlpha->SetCurrentValue(currentMaxAlpha);
        }
    } else if (widget == scrollBarFadeSpeed)
    {
        // update the scroll and progress bar
        float value = scrollBarFadeSpeed->GetCurrentValue();
        progressBarFadeSpeed->SetCurrentValue(value);
        configWidget->SetFadeSpeed(value);

        pct.Format("%.0f%%",value/0.10f);
        textFadeSpeedPct->SetText(pct);
    }
    return true;
}


bool WidgetConfigWindow::OnButtonPressed( int mouseButton, int keyModifier, pawsWidget* widget )
{
    if (widget == buttonOK)
    {
        // store settings
        configWidget->SetMinAlpha((int)currentMinAlpha);
        configWidget->SetMaxAlpha((int)currentMaxAlpha);	

        // close this window
        //windowManager->GetMainWidget()->DeleteChild(this);
        Hide();
        return false;
    } 
    else if (widget == buttonCancel)
    {
        // restore settings
        configWidget->SetMinAlpha((int)oldMinAlpha);
        configWidget->SetMaxAlpha((int)oldMaxAlpha);
        configWidget->SetFadeSpeed(oldFadeSpeed);
        configWidget->SetFade(oldFadeStatus);

        // close this window
        //windowManager->GetMainWidget()->DeleteChild(this);
        Hide();
        return false;
    }
    else if (widget ==buttonFade)
    {
        // enables/disables fading
        bool newFadeStatus = !configWidget->isFadeEnabled();
        configWidget->SetFade(newFadeStatus);
        buttonFade->SetState(newFadeStatus);

        if (buttonFade->GetState())
            textFadeStatus->SetText("Enabled");
        else
            textFadeStatus->SetText("Disabled");
    }
    return true;
}

void WidgetConfigWindow::SetConfigurableWidget(pawsWidget *configWidget)
{
    // get new values
    currentMinAlpha   = configWidget->GetMinAlpha();
    currentMaxAlpha   = configWidget->GetMaxAlpha();
    currentFadeStatus = configWidget->isFadeEnabled();
    currentFadeSpeed  = configWidget->GetFadeSpeed();

    // set title ("Window Settings (widgetname)"
    psString title;
    title = "Window Settings (";
    title += configWidget->GetName();
    title += ")";
    this->SetTitle(title);

    // store old values
    oldMinAlpha = currentMinAlpha;
    oldMaxAlpha = currentMaxAlpha;
    oldFadeStatus = currentFadeStatus;
    oldFadeSpeed  = currentFadeSpeed;

    scrollBarMinAlpha->SetMaxValue(255);
    scrollBarMinAlpha->SetCurrentValue(currentMinAlpha);
    scrollBarMinAlpha->SetTickValue(20.0);

    progressBarMinAlpha->SetTotalValue(255);
    progressBarMinAlpha->SetCurrentValue(currentMinAlpha);

    scrollBarMaxAlpha->SetMaxValue(255);
    scrollBarMaxAlpha->SetCurrentValue(currentMaxAlpha);
    scrollBarMaxAlpha->SetTickValue(20.0);

    progressBarMaxAlpha->SetTotalValue(255);
    progressBarMaxAlpha->SetCurrentValue(currentMaxAlpha);

    scrollBarFadeSpeed->SetMaxValue(10);
    scrollBarFadeSpeed->SetCurrentValue(currentFadeSpeed);
    scrollBarFadeSpeed->SetTickValue(0.4F);

    progressBarFadeSpeed->SetTotalValue(10);
    progressBarFadeSpeed->SetCurrentValue(currentFadeSpeed);

    psString pct;
    pct.Format("%.0f%%",currentMinAlpha/2.55f);
    textMinAlphaPct->SetText(pct);

    pct.Format("%.0f%%",currentMaxAlpha/2.55f);
    textMaxAlphaPct->SetText(pct);

    pct.Format("%.0f%%",currentFadeSpeed/0.10f);
    textFadeSpeedPct->SetText(pct);

    buttonFade->SetState(configWidget->isFadeEnabled());

    if (buttonFade->GetState())
        textFadeStatus->SetText("Enabled");
    else
        textFadeStatus->SetText("Disabled");

	this->configWidget = configWidget;
}


bool WidgetConfigWindow::PostSetup()
{
    if ((scrollBarMinAlpha   =(pawsScrollBar*  )this->FindWidget("MinAlphaScroll")) == NULL) return false;
    if ((progressBarMinAlpha =(pawsProgressBar*)this->FindWidget("MinAlphaProgressBar")) == NULL) return false;

    if ((scrollBarMaxAlpha   =(pawsScrollBar*  )this->FindWidget("MaxAlphaScroll")) == NULL) return false;
    if ((progressBarMaxAlpha =(pawsProgressBar*)this->FindWidget("MaxAlphaProgressBar")) == NULL) return false;

    if ((scrollBarFadeSpeed	  =(pawsScrollBar*  )this->FindWidget("FadeSpeedScroll")) == NULL) return false;
    if ((progressBarFadeSpeed =(pawsProgressBar*)this->FindWidget("FadeSpeedProgressBar")) == NULL) return false;
	
    if ((buttonOK     =(pawsButton*)this->FindWidget("OKButton")) == NULL) return false;
    if ((buttonCancel =(pawsButton*)this->FindWidget("CancelButton")) == NULL) return false;
    if ((buttonFade   =(pawsButton*)this->FindWidget("FadeButton")) == NULL) return false;
	
    if ((textFadeStatus   =(pawsTextBox*)this->FindWidget("FadeStatus")) == NULL) return false;
    if ((textMinAlphaPct  =(pawsTextBox*)this->FindWidget("MinAlphaCurrentPct")) == NULL) return false;
    if ((textMaxAlphaPct  =(pawsTextBox*)this->FindWidget("MaxAlphaCurrentPct")) == NULL) return false;
    if ((textFadeSpeedPct =(pawsTextBox*)this->FindWidget("FadeSpeedCurrentPct")) == NULL) return false;

    // place window on current mouse position
    MoveTo(windowManager->GetMouse()->GetPosition().x,
           windowManager->GetMouse()->GetPosition().y);

    return true;
}

